/**    
 * 文件名：CueListResponse.java    
 *    
 * 版本信息：    
 * 日期：2016年3月10日    
 * Copyright to PythagorasTech Corporation 2016     
 * 版权所有    
 *    
 */
package com.miaozhen.device.response;

import com.miaozhen.device.model.Cue;
import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParserFactory;
import java.io.IOException;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

/**
 * 项目名称：SmartTMS_S3
 * 类名称：CueListResponse
 * 类描述：
 * 创建人：mayp
 * 创建时间：2016年3月10日 上午10:38:37
 * 修改人：mayp
 * 修改时间：2016年3月10日 上午10:38:37
 * 修改备注：
 * 
 * @version
 */
public class CueListResponse extends Response {
    protected List<Cue> cueList;

    /**
     * 创建一个新的实例 CueListResponse.
     * 
     * @param string
     * @throws Exception
     */
    public CueListResponse(String string,final String element, final boolean isDolbyServer) throws Exception {
        super(string);
        handler = new DefaultHandler() {
            private String tag = null;

            private String uuid;

            @Override
            public void startElement(String uri, String localName, String qName, Attributes attributes) throws SAXException {
                if (qName.equals(element)) {
                    tag = qName;
                    uuid = attributes.getValue("uuid");
                }
            }

            @Override
            public void endElement(String uri, String localName, String qName) throws SAXException {
                if (qName.equals(element)) {
                    tag = null;
                }
            }

            @Override
            public void characters(char[] ch, int start, int length) throws SAXException {
                if (tag != null && tag.equals(element)) {
                    Cue cue = new Cue();
                    String cueLabel = new String(ch, start, length);
                    cue.setCueLabel(cueLabel);
                    cue.setCueName(cueLabel);
                    if(uuid == null) {
                        uuid = "urn:uuid:" + UUID.randomUUID();
                    }else{
                        if(isDolbyServer) {
                            uuid = uuid.substring("urn:uuid:".length());
                        }
                    }
                    cue.setCueUUID(uuid);
                    cueList.add(cue);
                }
            }
        };
        try {
            cueList = new ArrayList<Cue>();
            parser = SAXParserFactory.newInstance().newSAXParser();
            parser.parse(new InputSource(new StringReader(string)), handler);
        } catch (ParserConfigurationException e) {
            // logger.error(e.toString());
            throw new Exception("ParserConfigurationException", e);
        } catch (SAXException e) {
            // logger.error(e.toString());
            throw new Exception("SAXException", e);
        } catch (IOException e) {
            // logger.error(e.toString());
            throw new Exception("IOException", e);
        } finally {
            handler = null;
            parser = null;
        }
    }

    public List<Cue> getCueList() {
        return cueList;
    }
}
