package com.miaozhen.device.response;


import com.pythagorastech.tms.device.dcserver.response.Response;
import com.pythagorastech.tms.device.dcserver.smi.IngestStatus;
import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParserFactory;
import java.io.IOException;
import java.io.StringReader;


public class IngestStatusResponse extends Response {

	protected IngestStatus ingestStatus = new IngestStatus();

	public IngestStatusResponse(String string) throws Exception {
		super(string);
		handler = new DefaultHandler() {
			private String tag = null;

			@Override
			public void startElement(String uri, String localName,
					String qName, Attributes attributes) throws SAXException {
				if (qName.equals("status") || qName.equals("percent")
						|| qName.equals("description")) {
					tag = qName;
				}
			}

			@Override
			public void endElement(String uri, String localName, String qName)
					throws SAXException {
				if (qName.equals("status") || qName.equals("percent")
						|| qName.equals("description")) {
					tag = null;
				}
			}

			@Override
			public void characters(char[] ch, int start, int length)
					throws SAXException {
				if (tag != null && tag.equals("status")) {
					String status = new String(ch, start, length);
					if (status.equalsIgnoreCase("RUNNING"))
						ingestStatus.setStatus(IngestStatus.STATUS_RUNNING);
					else if (status.equalsIgnoreCase("STOPPED"))
						ingestStatus.setStatus(IngestStatus.STATUS_STOPPED);
					else if (status.equalsIgnoreCase("Canceled"))
						ingestStatus.setStatus(IngestStatus.STATUS_CANCEL);
					else if (status.equalsIgnoreCase("QUEUED"))
						ingestStatus.setStatus(IngestStatus.STATUS_QUEUED);
					else if (status.equalsIgnoreCase("FINISHED"))
						ingestStatus.setStatus(IngestStatus.STATUS_COMPLETE);
					else 
						ingestStatus.setStatus(IngestStatus.STATUS_ERROR);
				} else if (tag != null && tag.equals("percent")) {
					ingestStatus.setPercent(Integer.parseInt(new String(ch,
							start, length)));
				} else if (tag != null && tag.equals("description")) {
					ingestStatus.setDescription(new String(ch, start, length));
				}
			}
		};
		try {
			parser = SAXParserFactory.newInstance().newSAXParser();
			parser.parse(new InputSource(new StringReader(string)), handler);
		} catch (ParserConfigurationException e) {
			//logger.error(e.toString());
			throw new Exception("ParserConfigurationException", e);
		} catch (SAXException e) {
			//logger.error(e.toString());
			throw new Exception("SAXException", e);
		} catch (IOException e) {
			//logger.error(e.toString());
			throw new Exception("IOException", e);
		} finally {
			handler = null;
			parser = null;
		}
	}

	public IngestStatus getIngestStatus() {
		return ingestStatus;
	}
}
