package com.miaozhen.device.response;

import com.pythagorastech.tms.device.dcserver.response.Response;
import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParserFactory;
import java.io.IOException;
import java.io.StringReader;

public class ServerInfoResponse extends Response {
	protected String model;
	protected String serial;
	protected String os;
	protected String software;
	protected String firmware;

	public ServerInfoResponse(String string) throws Exception {
		super(string);
		handler = new DefaultHandler() {
			private String tag = null;

			@Override
			public void startElement(String uri, String localName,
					String qName, Attributes attributes) throws SAXException {
				if (qName.equals("model") || qName.equals("serial")) {
					tag = qName;
				} else if (qName.equals("version")) {
					os = attributes.getValue("os");
					software = attributes.getValue("software");
					firmware = attributes.getValue("firmware");
				}
			}

			@Override
			public void endElement(String uri, String localName, String qName)
					throws SAXException {
				if (qName.equals("model") || qName.equals("serial")
						|| qName.equals("version")) {
					tag = null;
				}
			}

			@Override
			public void characters(char[] ch, int start, int length)
					throws SAXException {
				if (tag != null) {
					if (tag.equals("model")) {
						model = new String(ch, start, length);
					} else if (tag.equals("serial")) {
						serial = new String(ch, start, length);
					}
				}
			}
		};
		try {
			parser = SAXParserFactory.newInstance().newSAXParser();
			parser.parse(new InputSource(new StringReader(string)), handler);
		} catch (ParserConfigurationException e) {
			//logger.error(e.toString());
			throw new Exception("ParserConfigurationException", e);
		} catch (SAXException e) {
			//logger.error(e.toString());
			throw new Exception("SAXException", e);
		} catch (IOException e) {
			//logger.error(e.toString());
			throw new Exception("IOException", e);
		} finally {
			handler = null;
			parser = null;
		}
	}

	public String getModel() {
		return model;
	}

	public String getSerial() {
		return serial;
	}

	public String getOSVersion() {
		return os;
	}

	public String getSoftwareVersion() {
		return software;
	}

	public String getFirmwareVersion() {
		return firmware;
	}

}
