package com.miaozhen.device.response;

import com.pythagorastech.tms.device.dcserver.smi.Schedule;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import java.io.IOException;
import java.io.StringReader;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;


public class SonyScheduleResponse {

    //	protected static org.slf4j.Logger logger = LoggerFactory
    //			.getLogger(SonyScheduleResponse.class);
    protected Schedule schedule;

    protected SAXParser parser;

    protected DefaultHandler handler;

    private Logger logger = LoggerFactory.getLogger(getClass());

    public SonyScheduleResponse(String string) throws Exception {
        handler = new DefaultHandler() {
            private String tag = null;
            private String show_uuid;
            private String iso_date_time;
            private String scheduleID;
            private String timestamp = null;
            private SimpleDateFormat timeFormat = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss");

            @Override
            public void startElement(String uri, String localName, String qName,
                Attributes attributes) throws SAXException {
                if (qName.equals("Timestamp") || qName.equals("ShowScheduleItem") || qName
                    .equals("ScheduleId") || qName.equals("PlayTime") || qName.equals("ShowId")) {
                    tag = qName;
                }
            }

            @Override
            public void endElement(String uri, String localName, String qName) throws SAXException {
                if (qName.equals("ShowScheduleItem")) {
                    schedule = new Schedule(scheduleID, show_uuid, iso_date_time);
                    logger.debug("endElement() timestamp: {}", timestamp);
                    if (timestamp != null) {
                        String str =
                            timeFormat.format(new Date(Long.valueOf(timestamp).longValue()));
                        try {
                            Date serverTime = timeFormat.parse(str);
                            schedule.setServerTime(serverTime);
                        } catch (ParseException e) {
                            e.printStackTrace();
                        }
                    }
                    tag = null;
                } else if (qName.equals("ScheduleId") || qName.equals("PlayTime") || qName
                    .equals("ShowId") || qName.equals("Timestamp")) {
                    tag = null;
                }
            }

            @Override
            public void characters(char[] ch, int start, int length) throws SAXException {
                if (tag != null && tag.equals("Timestamp")) {
                    timestamp = new String(ch, start, length);
                } else if (tag != null && tag.equals("ScheduleId")) {
                    scheduleID = new String(ch, start, length);
                } else if (tag != null && tag.equals("ShowId")) {
                    show_uuid = new String(ch, start, length);
                } else if (tag != null && tag.equals("PlayTime")) {
                    iso_date_time = new String(ch, start, length);
                }
            }
        };
        try {
            parser = SAXParserFactory.newInstance().newSAXParser();
            parser.parse(new InputSource(new StringReader(string)), handler);
        } catch (ParserConfigurationException e) {
            //logger.error(e.toString());
            throw new Exception("ParserConfigurationException", e);
        } catch (SAXException e) {
            //logger.error(e.toString());
            throw new Exception("SAXException", e);
        } catch (IOException e) {
            //logger.error(e.toString());
            throw new Exception("IOException", e);
        } finally {
            handler = null;
            parser = null;
        }
    }

    public Schedule getSchedule() {
        return schedule;
    }

    public static void main(String[] args) {
        String xml = "<ShowScheduleItem> "
            + "<ScheduleId>urn:uuid:61849710-c384-4a47-919f-b68fcbf8c365</ScheduleId> "
            + "<PlayTime>2012-08-26T14:31:32.000-12:00</PlayTime> "
            + "<ShowId>urn:uuid:fa2d34c8-26e5-4f23-b0e5-60bccc80e86e</ShowId> "
            + "<WaitForTrigger>false</WaitForTrigger> "
            + "<NextShowState>PENDING_START</NextShowState> " + "</ShowScheduleItem> ";
        try {
            com.pythagorastech.tms.device.dcserver.response.SonyScheduleResponse response = new com.pythagorastech.tms.device.dcserver.response.SonyScheduleResponse(xml);
            Schedule sch = response.getSchedule();
            System.out
                .println(sch.getShowUUID() + "   " + sch.getISODateTime() + "   " + sch.getUUID());
        } catch (Exception e) {
            e.printStackTrace();
        }

    }
}
