package com.miaozhen.device.support;

import com.barco.dual.projector.EPlatform;
import com.barco.dual.projector.OSinfo;
import com.miaozhen.device.IDCServer;
import com.miaozhen.device.ServerFactory;
import com.miaozhen.device.expection.DeviceException;
import com.miaozhen.device.response.PlaybackStatusResponse;
import com.miaozhen.device.response.Response;
import com.miaozhen.device.smi.PlaybackStatus;
import com.miaozhen.device.support.christienew.SystemManagementChrisNew;
import com.miaozhen.device.support.christienew.basecmd.UtilTMSChrisNew;
import com.miaozhen.device.support.christienew.playbackcontrol.PlaybackControlChrisNew;
import com.pythagorastech.tms.device.dcserver.response.StringResponse;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.*;
import java.net.Socket;
import java.net.SocketException;

public class ChristieFusionServer implements IDCServer {

    private String ip;

    protected SystemManagementChrisNew systemManagement;

    protected PlaybackControlChrisNew playbackControl;



    private Socket socket = null;

    private UtilTMSChrisNew util = new UtilTMSChrisNew();

    private final Logger logger = LoggerFactory.getLogger(getClass());

    private final Logger deviceLog = LoggerFactory.getLogger("device");

    private boolean isConnected = false;

    public ChristieFusionServer(String ip, int port, String userName, String password) {
        this.ip = ip;
        isConnected = checkConnect();
        if (isConnected) {
            this.socket = new Socket();
            util.login(this.socket, ip, 5001);
            try {
                socket.setSoTimeout(60000);
            } catch (SocketException e) {
                logger.info("ChristieFusionServer connect err",e);
            }
        }
        systemManagement = new SystemManagementChrisNew();
        playbackControl = new PlaybackControlChrisNew();

    }

    public boolean checkConnect() {
        EPlatform OSInfo = OSinfo.getOSname();
        Runtime runtime = Runtime.getRuntime();
        Process process = null;
        try {
            if (OSInfo.equals(EPlatform.Windows)) {
                process = runtime.exec("ping " + this.ip + " -n 1 -w 3");

            } else {
                process = runtime.exec("ping " + this.ip + " -c 1 -w 3");

            }
            InputStream is = process.getInputStream();
            InputStreamReader isr = new InputStreamReader(is);
            BufferedReader br = new BufferedReader(isr);
            String line = null;
            StringBuffer sb = new StringBuffer();
            while ((line = br.readLine()) != null) {
                sb.append(line);
            }
            is.close();
            isr.close();
            br.close();
            if (null != sb && !sb.toString().equals("")) {
                if (sb.toString().toLowerCase().indexOf("ttl") > 0) {
                    return true;
                } else {
                    return false;
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        } finally {
            if (process != null)
                process.destroy();
        }
        return false;
    }

    @Override
    public void setConnected(boolean connected) {
        isConnected = connected;
    }

    @Override
    public boolean getConnected(){
        return isConnected;
    }

    @Override
    public void disconnect() throws DeviceException {

    }

    @Override
    public void connect() throws DeviceException {
        if (!isConnected) {
            isConnected = checkConnect();
        }
        if (isConnected) {
            if (socket == null || socket.isClosed()) {
                socket = new Socket();
            }
            if (util.login(this.socket, ip, 5001)) {
                try {
                    socket.setSoTimeout(60000);
                } catch (SocketException e) {
                    logger.info("ChristieFusionServer connect err",e);
                    throw new DeviceException();
                }
                systemManagement.setSocket(socket);
            } else {
                socket = new Socket();
                isConnected = false;
            }
        }
    }

    @Override
    public boolean ejectSPL(String uuid) throws Exception {
        return false;
    }

    @Override
    public String getServerType() {
        return ServerFactory.ServerType.ChristieFusion.toString();
    }

    @Override
    public String getIP() {
        return ip;
    }

    @Override
    public void setIP(String ip) {
        this.ip = ip;
    }

    @Override
    public String getAuditLog(String ftpIp, String user, String password, String tempLogPath,
                              String startDate, String startTime, String endDate, String endTime) throws Exception {
        if (isConnected) {
            logger.info("keshi ChristieServer getAuditLog begin ip is:{}",ip);
            String xml = systemManagement.getauditlogs(ip, tempLogPath, startDate, startTime, endDate, endTime, "", "");
            logger.info("keshi ChristieServer getAuditLog ip is:{}, result : {}",ip, xml);
            StringResponse response = new StringResponse(xml, "auditlog_path");
            if (response.status() == com.pythagorastech.tms.device.dcserver.response.Response.OK) {
                File xmlFile = new File(response.getResponse());
                StringBuffer sb = new StringBuffer();
                FileReader reader = new FileReader(xmlFile);
                BufferedReader br = new BufferedReader(reader);
                String str = null;
                while ((str = br.readLine()) != null) {
                    sb.append(str);
                }
                try {
                    xmlFile.deleteOnExit();
                } catch (Exception e) {
                    logger.error("delete file error", e);
                }
                return sb.toString();
            }
        }
        return null;
    }

    @Override
    public PlaybackStatus getPlaybackStatus() throws Exception {
        if (isConnected) {
//            logger.info("playbackControl is:{},ip is:{}",playbackControl,ip);
            if(playbackControl.getSocket()==null){
                playbackControl.setSocket(socket);
            }
            String xml = playbackControl.getplaybackstatus(ip);
//            logger.info("ChristieServer getPlaybackStatus, result : {}", xml);
            PlaybackStatusResponse response = new PlaybackStatusResponse(xml);
            if(response.getPlaybackStatus().getState() == -1){
                for (int i = 0; i < 3; i++) {
                    logger.info("getPlaybackStatus retry {}/3",i+1);
                    xml = playbackControl.getplaybackstatus(ip);
                    response = new PlaybackStatusResponse(xml);
                    if (response.status() == Response.OK && response.getPlaybackStatus().getState() != -1)
                        break;
                }
            }

            if (response.status() == Response.OK) {
                return response.getPlaybackStatus();
            }
        }
        return null;
    }
}
