package com.miaozhen.device.support;

import com.barco.dual.projector.EPlatform;
import com.barco.dual.projector.OSinfo;
import com.miaozhen.device.support.christienew.*;
import com.miaozhen.device.support.christienew.basecmd.UtilTMSChrisNew;
import com.miaozhen.util.ExpiringCacheUtil;
import com.pythagorastech.tms.device.dcserver.IDCServer;
import com.pythagorastech.tms.device.dcserver.response.*;
import com.pythagorastech.tms.device.dcserver.smi.*;
import com.pythagorastech.tms.device.dcserver.support.smi.CrifstSPL_2012;
import com.pythagorastech.tms.device.dcserver.util.LogProcess;
import com.pythagorastech.tms.device.expection.DeviceException;
import com.pythagorastech.tms.util.DateUtil;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.net.ftp.FTPClient;
import org.apache.commons.net.ftp.FTPReply;
import org.crifst.project.RM.security.certificate.DcCertReader;
import org.dom4j.DocumentHelper;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.input.SAXBuilder;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.*;
import java.net.Socket;
import java.net.SocketException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

public class ChristieS4Server implements IDCServer {

    private String ip;

    protected ContentManagementChrisNew contentManagement;

    protected LicenseManagementChrisNew licenseManagement;

    protected ShowManagementChrisNew showManagement;

    protected SystemManagementChrisNew systemManagement;

    protected TransferManagementChrisNew transferManagement;

    protected PlaybackControlChrisNew playbackControl;

    private Socket socket = null;

    private UtilTMSChrisNew util = new UtilTMSChrisNew();

    private final Logger logger = LoggerFactory.getLogger(getClass());

    private final Logger deviceLog = LoggerFactory.getLogger("device");

    private boolean isConnected = false;

    ExpiringCacheUtil<String,Object> cacheUtil = new ExpiringCacheUtil<>(20);

    public ChristieS4Server(String ip, int port, String userName, String password) {
        this.ip = ip;
        isConnected = checkConnect();
        if (isConnected) {
            this.socket = new Socket();
            util.login(this.socket, ip, 5001);
            try {
                socket.setSoTimeout(3000);
            } catch (SocketException e) {
                logger.info("ChristieFusionServer connect err",e);
            }
        }
        contentManagement = new ContentManagementChrisNew();//111
        licenseManagement = new LicenseManagementChrisNew();//111
        showManagement = new ShowManagementChrisNew();//111
        systemManagement = new SystemManagementChrisNew();//111
        transferManagement = new TransferManagementChrisNew();//11
        playbackControl = new PlaybackControlChrisNew();

    }

    @Override
    public String getIP() {
        return ip;
    }

    @Override
    public void setIP(String ip) {
        this.ip = ip;
    }

    public boolean checkConnect() {
        EPlatform OSInfo = OSinfo.getOSname();
        Runtime runtime = Runtime.getRuntime();
        Process process = null;
        try {
            if (OSInfo.equals(EPlatform.Windows)) {
                process = runtime.exec("ping " + this.ip + " -n 1 -w 3");

            } else {
                process = runtime.exec("ping " + this.ip + " -c 1 -w 3");

            }
            InputStream is = process.getInputStream();
            InputStreamReader isr = new InputStreamReader(is);
            BufferedReader br = new BufferedReader(isr);
            String line = null;
            StringBuffer sb = new StringBuffer();
            while ((line = br.readLine()) != null) {
                sb.append(line);
            }
            is.close();
            isr.close();
            br.close();
            if (null != sb && !sb.toString().equals("")) {
                if (sb.toString().toLowerCase().indexOf("ttl") > 0) {
                    return true;
                } else {
                    return false;
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        } finally {
            if (process != null)
                process.destroy();
        }
        return false;
    }

    @Override
    public void setConnected(boolean connected) {
        isConnected = connected;
    }

    @Override
    public void connect() throws DeviceException {
        if (!isConnected) {
            isConnected = checkConnect();
        }
        if (isConnected) {
            if (socket == null || socket.isClosed()) {
                socket = new Socket();
            }
            if (util.login(this.socket, ip, 5001)) {
                try {
                    socket.setSoTimeout(3000);
                } catch (SocketException e) {
                    logger.info("ChristieFusionServer connect err",e);
                    throw new DeviceException();
                }
                contentManagement.setSocket(socket);
                licenseManagement.setSocket(socket);
                showManagement.setSocket(socket);
                systemManagement.setSocket(socket);
                transferManagement.setSocket(socket);
                playbackControl.setSocket(socket);
            } else {
                socket = new Socket();
                isConnected = false;
            }
        }
    }

    @Override
    public void disconnect() throws DeviceException {

    }

    public static String handleuuid(String uuid){
    	String header="urn:uuid:";
    	if(uuid.contains(header)){
    		String fin=uuid.substring(9, uuid.length());
    		return fin;
    	}else{
    		return uuid;
    	}
    }
    
    @Override
    public String getCPL(String uuid) throws Exception {
        if (isConnected) {
            if(cacheUtil.get("cpl_"+uuid)!=null){
                return (String)cacheUtil.get("cpl_"+uuid);
            }else{
                String cplXML = contentManagement.getcpl(ip, uuid);
                deviceLog.trace("ChristieServer getCPL, uuid : {}, result : {}", uuid, cplXML);
                XMLResponse response = new XMLResponse(cplXML);
                if (response.status() == Response.OK) {
                    if(StringUtils.isNotBlank(response.getXML())){
                        cacheUtil.put("cpl_"+uuid,response.getXML());
                    }
                    return response.getXML();
                }
            }
        }
        return null;
    }

    //yigai
    @Override
    public List<String> getCPLList() throws Exception {
        if (isConnected) {
            if(cacheUtil.get("cplList_")!=null){
                return (List<String>)cacheUtil.get("cplList_");
            }else{
                String xml = contentManagement.getcpllist(ip);
                deviceLog.trace("ChristieServer getCPLList, result : {}", xml);
                ListResponse response = new ListResponse(xml, "cpl_uuid");
                if (response.status() == Response.OK){
                    if(response.getList() == null){
                        logger.info("ChristieServer getCPLList null xml is:{}",xml);
                    }
                    List<String> list=response.getList();
                    for(int i=0;i<list.size();i++){
                        String str=list.get(i);
                        str="urn:uuid:"+str;
                        list.set(i, str);
                    }
                    if(list!=null&&list.size()>0){
                        cacheUtil.put("cplList_",list);
                    }
                    return list;
                }
            }
        }
        return null;
    }

  //yigai
    @Override
    public boolean deleteContent(String uuid) throws Exception {
        if (isConnected) {
            String xml = contentManagement.deletecontent(ip, uuid);
            deviceLog.trace("ChristieServer deleteContent, uuid : {}, result : {}", uuid, xml);
            Response response = new Response(xml);
            if (response.status() == Response.OK)
                return true;
        }
        return false;
    }

    @Override
    public boolean validateCPL(String s) throws Exception {
        return false;
    }

    //yigai
    @Override
    public String getKDM(String uuid) throws Exception {
        if (isConnected) {
            String xml = licenseManagement.getkdm(ip, uuid);
            deviceLog.trace("ChristieServer getKDM, uuid : {}, result : {}", uuid, xml);
            XMLResponse response = new XMLResponse(xml);
            if (response.status() == Response.OK)
                return response.getXML();
        }
        return null;
    }

  //yigai
    @Override
    public List<String> getKDMList() throws Exception {
        if (isConnected) {
            if(cacheUtil.get("kdmList_")!=null){
                return (List<String>)cacheUtil.get("kdmList_");
            }else{
                String xml = licenseManagement.getkdmlist(ip);
                deviceLog.trace("ChristieServer getKDMList, result : {}", xml);
                ListResponse response = new ListResponse(xml, "asset_uuid");
                if (response.status() == Response.OK) {
                    if(response.getList() == null){
                        logger.info("ChristieServer getKDMList null xml is:{}",xml);
                    }
                    List<String> list = response.getList();
                    if(list!=null&&list.size()>0){
                        cacheUtil.put("kdmList_",list);
                    }
                    return list;
                }
            }
        }
        return null;
    }

    //yigai
    @Override
    public boolean deleteKDM(String uuid) throws Exception {
        if (isConnected) {
            String xml = licenseManagement.deletekdm(ip, uuid);
            deviceLog.trace("ChristieServer deleteKDM, uuid : {}, result : {}", uuid, xml);
            if (xml == null || xml.equals("ERROR")) {
                xml = licenseManagement.deletekdm(ip, uuid);
            }
            Response response = new Response(xml);
            if (response.status() == Response.OK)
                return true;
        }
        return false;
    }

    //yigai
    @Override
    public PlaybackStatus getPlaybackStatus() throws Exception {
        if (isConnected) {
            String xml = playbackControl.getplaybackstatus(ip);
//            logger.info("ChristieServer getPlaybackStatus, result : {}", xml);
            PlaybackStatusResponse response = new PlaybackStatusResponse(xml);
            if(response.getPlaybackStatus().getState() == -1){
                for (int i = 0; i < 3; i++) {
                    logger.info("getPlaybackStatus retry {}/3",i+1);
                    xml = playbackControl.getplaybackstatus(ip);
                    response = new PlaybackStatusResponse(xml);
                    if (response.status() == Response.OK && response.getPlaybackStatus().getState() != -1)
                        break;
                }
            }

            if (response.status() == Response.OK) {
                return response.getPlaybackStatus();
            }
        }
        return null;
    }

    @Override
    public boolean playSPL() throws Exception {
        if (isConnected) {
            String xml = playbackControl.playspl(ip);
            deviceLog.trace("ChristieServer playSPL, result : {}", xml);
            Response response = new Response(xml);
            if (response.status() == Response.OK)
                return true;
        }
        return false;
    }

    @Override
    public boolean pauseSPL() throws Exception {
        if (isConnected) {
            String xml = playbackControl.pausespl(ip);
            deviceLog.trace("ChristieServer pauseSPL, result : {}", xml);
            Response response = new Response(xml);
            if (response.status() == Response.OK)
                return true;
        }
        return false;
    }

    @Override
    public boolean stopSPL() throws Exception {
        if (isConnected) {
            String xml = playbackControl.stopspl(ip);
            deviceLog.trace("ChristieServer stopSPL, result : {}", xml);
            Response response = new Response(xml);
            if (response.status() == Response.OK)
                return true;
        }
        return false;
    }

    @Override
    public boolean next() throws Exception {
        if (isConnected) {
            String xml = playbackControl.next(ip);
            deviceLog.trace("ChristieServer next, result : {}", xml);
            Response response = new Response(xml);
            if (response.status() == Response.OK)
                return true;
        }
        return false;
    }

    @Override
    public boolean previous() throws Exception {
        if (isConnected) {
            String xml = playbackControl.previous(ip);
            deviceLog.trace("ChristieServer previous, result : {}", xml);
            Response response = new Response(xml);
            if (response.status() == Response.OK)
                return true;
        }
        return false;
    }

    @Override
    public boolean gotoPosition(int offset) throws Exception {
        if (isConnected) {
            String xml = playbackControl.gotoposition(ip, offset);
            deviceLog.trace("ChristieServer gotoPosition, offset : {}, result : {}", offset, xml);
            Response response = new Response(xml);
            if (response.status() == Response.OK)
                return true;
        }
        return false;
    }

    @Override
    public boolean selectSPL(String uuid) throws Exception {
        if (isConnected) {
            String xml = playbackControl.selectspl(ip, uuid);
            deviceLog.trace("ChristieServer selectSPL, uuid : {}, result : {}", uuid, xml);
            Response response = new Response(xml);
            if (response.status() == Response.OK)
                return true;
        }
        return false;
    }

    @Override
    public List<String> getSPLList() throws Exception {
        if (isConnected) {
            if(cacheUtil.get("splList_")!=null){
//                logger.debug("getSPLList ip is:{} catsplList is:{}",ip,JsonMapper.getInstance().toJson((List<String>)cacheUtil.get("splList_")));
                return (List<String>)cacheUtil.get("splList_");
            }else{
                String xml = showManagement.getspllist(ip);
                deviceLog.trace("ChristieServer getSPLList, result : {}", xml);
                ListResponse response = new ListResponse(xml, "show_uuid");
                if (response.status() == Response.OK) {
                    List<String> list = response.getList();
                    if(list!=null&&list.size()>0){
//                        logger.debug("getSPLList ip is:{} getSPLList is:{}",ip,JsonMapper.getInstance().toJson(list));
                        cacheUtil.put("splList_",list);
                    }
                    return list;
                }
            }
        }
        return null;
    }

    @Override
    public String getSPL(String uuid) throws Exception {
        if (isConnected) {
            if(cacheUtil.get("spl_"+uuid)!=null){
                return (String)cacheUtil.get("spl_"+uuid);
            }else{
                String xml = showManagement.getpartialspl(ip, uuid);
                deviceLog.trace("ChristieServer getSPL, uuid : {}, result : {}", uuid, xml);
                if (xml == null || !xml.contains("CDSPlaylist")) {
                    return null;
                }
                XMLResponse response = new XMLResponse(xml);
                String tmsXML = CrifstSPL_2012.christieSPLToCrifstSPL(response.getXML());
                if (response.status() == Response.OK) {
                    if (StringUtils.isNotBlank(xml)) {
                        cacheUtil.put("spl_" + uuid, xml);
                    }
                    return tmsXML;
                }
            }
        }
        return null;
    }

    @Override
    public boolean validateSPL(String s) throws Exception {
        return false;
    }

    @Override
    public boolean deleteSPL(String uuid) throws Exception {
        if (isConnected) {
            String xml = showManagement.deletespl(ip, uuid);
            deviceLog.trace("ChristieServer deleteSPL, uuid : {}, result : {}", uuid, xml);
            Response response = new Response(xml);
            if (response.status() == Response.OK)
                return true;
        }
        return false;
    }

    @Override
    public String createSchedule(String s, String s1) throws Exception {
        return null;
    }

    @Override
    public List<Schedule> getScheduleList(String scheduleadate) throws Exception {
        if (isConnected) {
            String xml = showManagement.getschedulelist(ip);
            deviceLog.trace("ChristieServer getScheduleList, scheduleadate : {}, result : {}",
                    scheduleadate, xml);
            ScheduleListResponse response = new ScheduleListResponse(xml);
            if (response.status() == Response.OK)
                return response.getScheduleList();
        }
        return null;
    }

    @Override
    public Schedule getSchedule(String uuid) throws Exception {
        return null;
    }

    @Override
    public boolean cancelSchedule(Schedule schedule) throws Exception {
        if (isConnected) {
            String xml = showManagement.cancelschedule(ip, schedule.getUUID());
            deviceLog.trace("ChristieServer cancelSchedule, result : {}", xml);
            Response response = new Response(xml);
            if (response.status() == Response.OK)
                return true;
        }
        return false;
    }

    @Override
    public boolean enableScheduler() throws Exception {
        if (isConnected) {
            String xml = showManagement.enablescheduler(ip);
            deviceLog.trace("ChristieServer enableScheduler, result : {}", xml);
            Response response = new Response(xml);
            if (response.status() == Response.OK)
                return true;
        }
        return false;
    }

    @Override
    public boolean disableScheduler() throws Exception {
        if (isConnected) {
            String xml = showManagement.disablescheduler(ip);
            deviceLog.trace("ChristieServer disableScheduler, result : {}", xml);
            Response response = new Response(xml);
            if (response.status() == Response.OK)
                return true;
        }
        return false;
    }

    @Override
    public boolean getSchedulerStatus() throws Exception {
        if (isConnected) {
            String xml = showManagement.getschedulerstatus(ip);
            deviceLog.trace("ChristieServer getSchedulerStatus, result : {}", xml);
            Response response = new Response(xml);
            if (response.status() == Response.OK) {
                if (xml.contains("TRUE")) {
                    return true;
                }
            }
        }
        return false;
    }

    @Override
    public String getServerDateTime() throws Exception {
        return null;
    }

    @Override
    public ServerInfo getServerInfo() throws Exception {
        return null;
    }

    @Override
    public StorageInfo getStorageInfo() throws Exception {
        return null;
    }

    @Override
    public String getPublickeyThumbprint() throws Exception {
        return null;
    }

    private String extractOristarpublickeythumbprint(String retVal) {
        try {
            SAXBuilder sb = new SAXBuilder();

            InputStream xmlInputStream = new ByteArrayInputStream(retVal.getBytes("UTF-8"));
            Document doc = sb.build(xmlInputStream);
            Element root = doc.getRootElement();
            String cert = root.getChildText("certificate");
            InputStream c = new ByteArrayInputStream(cert.getBytes());
            DcCertReader cerfreader = new DcCertReader(c);
            String publickeythumbprint = "";
            String subject = cerfreader.getSubject();
            String[] pairs = subject.split(",");
            for (String pair : pairs) {
                int cnPos = pair.indexOf("CN=");
                if (cnPos != -1) {
                    String cnString = pair.replace("CN=", "").replace("\134", "");
                    publickeythumbprint = cnString.substring(cnString.indexOf(".") + 1);
                }
            }
            return
                    "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"OK\" version=\"2\" >\n\t<publickeythumbprint>"
                            + publickeythumbprint + "</publickeythumbprint>" + "\n</response>";
        } catch (Exception e) {
            return
                    "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"ERROR\" version=\"2\" >\n\t<error>"
                            + e.getMessage() + "</error>" + "\n</response>";
        }
    }

    @Override
    public String ingestPackage(String host, String port, String user, String password,
                                String cpl_UUID, String cpl_path) throws Exception {
        if (isConnected) {
        	
        	String xml2 =transferManagement.ScanRemoteIngestibles(ip, host, user, password);
        	/*String xml1 =
                    transferManagement.listAvaliableContent(ip);*/
            String xml =
                    transferManagement.ingestpackage(ip, host, user, password, cpl_UUID, cpl_path);
            deviceLog.trace("ChristieServer ingestPackage, uuid : {}, result : {}", cpl_UUID, xml);
            StringResponse response = new StringResponse(xml, "ingest_uuid");
            if (response.status() == Response.OK)
                return response.getResponse();
        }
        return null;
    }

    @Override
    public String ingestKDM(String host, String port, String user, String password, String path,
                            String uuid) throws Exception {
        if (isConnected) {
            String kdm = getfilefromftp(host, user, password, path);
            String xml = transferManagement.ingestkdm(ip, kdm);
            deviceLog.trace("ChristieServer ingestKDM, uuid : {}, result : {}", uuid, xml);
            /*StringResponse response = new StringResponse(xml, "ingest_uuid");
            if (response.status() == Response.OK)
                return response.getResponse();*/
            if(xml.equals("ok")){
            	return "true";
            }
        }
        return "false";
    }

    @Override
    public IngestStatus getIngestStatus(String uuid, String importPath) throws Exception {
        if (isConnected) {
            String xml = transferManagement.getingeststatus(ip, uuid);
            deviceLog.trace("ChristieServer getIngestStatus, uuid : {}, result : {}", uuid, xml);
            //科视2.0.1   传输用UUID识别     2.1.0传输用 title识别
            if(xml.contains("UUID NOT FOUND")){
                xml = transferManagement.getingeststatus(ip, importPath);
            }
            IngestStatusResponse response = new IngestStatusResponse(xml);
            if (response.status() == Response.OK)
                return response.getIngestStatus();
        }
        return null;
    }

    @Override
    public boolean cancelIngest(String uuid) throws Exception {
        logger.info("keylog cancelIngest param {}", uuid);
        String arr[] = uuid.split("\\|");
        String cplTitle = arr[0];
        String transferUUID = arr[1];

        if (isConnected) {
            String xml = transferManagement.cancelingest(ip, cplTitle);
            deviceLog.trace("ChristieServer cancelIngest, cplTitle : {}, result : {}", cplTitle, xml);
            if (xml == null || xml.equals("ERROR") || xml.contains("ERROR")) {
                xml = transferManagement.cancelingest(ip, transferUUID);
                try {
                    Socket socket = transferManagement.getSocket();
                    if (socket != null && !socket.isClosed()) {
                        socket.close();
                    }
                } catch (Throwable throwable) {
                    logger.error("cancelIngest throwable {}", throwable.getMessage());
                }
            }
            Response response = new Response(xml);
            if (response.status() == Response.OK)
                return true;
        }
        return false;
    }

    @Override
    public String getAuditLog(String ftpIp, String user, String password, String tempLogPath,
                              String startDate, String startTime, String endDate, String endTime) throws Exception {
        if (isConnected) {
            logger.info("keshi ChristieServer getAuditLog begin ip is:{}",ip);
            String xml = systemManagement.getauditlogs(ip, tempLogPath, startDate, startTime, endDate, endTime, "", "");
            logger.info("keshi ChristieServer getAuditLog ip is:{}, result : {}",ip, xml);
            StringResponse response = new StringResponse(xml, "auditlog_path");
            if (response.status() == Response.OK) {
                File xmlFile = new File(response.getResponse());
                StringBuffer sb = new StringBuffer();
                FileReader reader = new FileReader(xmlFile);
                BufferedReader br = new BufferedReader(reader);
                String str = null;
                while ((str = br.readLine()) != null) {
                    sb.append(str);
                }
                try {
                    xmlFile.deleteOnExit();
                } catch (Exception e) {
                    logger.error("delete file error", e);
                }
                return sb.toString();
            }
        }
        return null;
    }

    @Override
    //SMS.log
    public List<ServerLogEntity> parseAuditLog(String xml) throws Exception {
        //增加文件头 和 根元素
        StringBuffer sb = new StringBuffer();
        sb.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<root>\n");
        sb.append(xml);
        sb.append("</root>");
        org.dom4j.Document document = DocumentHelper.parseText(sb.toString());
        org.dom4j.Element element = document.getRootElement();
        List<org.dom4j.Element> list = element.elements();
        LogProcess logProcess = new LogProcess();
        for (org.dom4j.Element logElement : list) {
            // 查到一条记录
            // <log>...</log>
            if (!logElement.getName().equals("log"))
                continue;
            // 查到这条记录中的元素：
            // <time>...</time> 时间
            org.dom4j.Element timeElement = logElement.element("time");
            // <message>...</message> 事件
            org.dom4j.Element messageElement = logElement.element("message");
            // <origin>...</origin> 类型
            org.dom4j.Element originElement = logElement.element("origin");
            if (timeElement == null || messageElement == null || originElement == null)
                continue;
            // 匹配<origin>SMS Playback</origin>的记录
            if ("SMS Playback".equals(originElement.getText())) {
                // 匹配<message>CPL Playback START...</message> 或 <message>CPL Playback END...</message>
                boolean isStart = messageElement.getText().contains("CPL Playback START");
                boolean isEnd = messageElement.getText().contains("CPL Playback END");
                if (isStart || isEnd) {
                    // 从<message>...</message>中截取cpl的uuid
                    // CPL的uuid是如下格式的
                    // 8-4-4-4-12 for a total of 36 characters
                    int uuidIndex = messageElement.getText().indexOf("UUID:") + "UUID:".length();
                    String contentId =
                            "urn:uuid:" + messageElement.getText().substring(uuidIndex, uuidIndex + 36)
                                    .trim();
                    // 时间日期格式转换
                    // SMS.log: <time>2017-04-10 10:17:38.0000045</time>
                    // SM.log: <lr:TimeStamp>2016-07-20T16:09:40+08:00</lr:TimeStamp>
                    DateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSS");
                    DateFormat utcFormat = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ssXXX");
                    String timeStr = timeElement.getText();
                    Date date = format.parse(timeStr);
                    timeStr = utcFormat.format(date);
                    logProcess.add(contentId, isStart, DateUtil.getEastern8Time(timeStr));
                }
            }
        }
        return logProcess.process();
    }

    //SM.log
    public List<ServerLogEntity> parseSMLog(String xml) throws Exception {
        //增加命名空间
        StringBuffer sb = new StringBuffer();
        sb.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
                + "<lr:LogReport xmlns:dcml=\"http://www.smpte-ra.org/schemas/433/2008/dcmlTypes/\"\n"
                + "    xmlns:ds=\"http://www.w3.org/2000/09/xmldsig#\"\n"
                + "    xmlns:lr=\"http://www.smpte-ra.org/schemas/430-4/2008/LogRecord/\"\n"
                + "    xmlns:xs=\"http://www.w3.org/2001/XMLSchema\"\n"
                + "    xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\">");
        sb.append(xml);
        sb.append("</lr:LogReport>");
        org.dom4j.Document document = DocumentHelper.parseText(sb.toString());
        org.dom4j.Element element = document.getRootElement();
        List<org.dom4j.Element> list = element.elements();
        LogProcess logProcess = new LogProcess();
        for (org.dom4j.Element logRecordElement : list) {
            if (!logRecordElement.getName().equals("LogRecordElement"))
                continue;
            org.dom4j.Element head = logRecordElement.element("LogRecordHeader");
            org.dom4j.Element body = logRecordElement.element("LogRecordBody");
            if (head == null || body == null)
                continue;
            org.dom4j.Element eventType = head.element("EventType");
            if (eventType == null)
                continue;
            org.dom4j.Element eventSubType = body.element("EventSubType");
            if (eventSubType == null)
                continue;
            if ("Playout".equals(eventType.getText())) {
                String eventSubTypeText = eventSubType.getText();
                String contentId = head.element("contentId").getText();
                boolean isStart = "CPLStart".equals(eventSubTypeText);
                boolean isEnd = "CPLEnd".equals(eventSubTypeText);
                if (isStart || isEnd) {
                    String timeStr = head.element("TimeStamp").getText();
                    logProcess.add(contentId, isStart, DateUtil.getEastern8Time(timeStr));
                }
            }
        }
        return logProcess.process();
    }

    @Override
    public List<Cue> getCueList() throws Exception {
        if (isConnected) {
            if(cacheUtil.get("cueList_")!=null){
                return (List<Cue>)cacheUtil.get("cueList_");
            }else{
                String xml = showManagement.getcuelist(ip);
                deviceLog.trace("ChristieServer getCueList, result : {}", xml);
                if (xml != null) {
                    CueListResponse response = new CueListResponse(xml, "automation_label", false);
                    if (response.status() == Response.OK) {
                        List<Cue> cueList = response.getCueList();
                        if(cueList!=null&&cueList.size()>0){
                            cacheUtil.put("cueList_",cueList);
                        }
                        return cueList;
                    }
                }
            }
        }
        return null;
    }

    @Override
    public boolean executeCue(String cueLable) {
        return false;
    }

    @SuppressWarnings("deprecation")
    private String getfilefromftp(String host, String user, String password, String path)
            throws SocketException, IOException {
        String result = "";
        FTPClient client = new FTPClient();

        client.connect(host);
        client.login(user, password);
        int reply = client.getReplyCode();

        if (530 == reply) {
            client.disconnect();
            return result;
        }
        client.setFileType(2);
        reply = client.getReplyCode();
        if (!(FTPReply.isPositiveCompletion(reply))) {
            client.disconnect();
            return result;
        }
        client.setBufferSize(1024);
        String serverSystem = client.getSystemName();
        String hostsEncoding;
        if (serverSystem.equals("Windows_NT")) {
            hostsEncoding = "GBK";
        } else {
            hostsEncoding = "utf-8";
        }
        client.setControlEncoding(hostsEncoding);
        if (!(client.isConnected())) {
            return result;
        }
        InputStream ftpIn =
                client.retrieveFileStream(new String(path.getBytes(hostsEncoding), "iso-8859-1"));
        if (ftpIn == null)
            return result;
        byte[] buf = new byte[1024];
        int bufsize = 0;
        OutputStream ftpOut = new ByteArrayOutputStream();
        while ((bufsize = ftpIn.read(buf, 0, buf.length)) != -1) {
            ftpOut.write(buf, 0, bufsize);
        }

        result = ftpOut.toString();
        ftpOut.close();
        ftpIn.close();
        if (client.isConnected()) {
            client.completePendingCommand();
        }
        return result;
    }

    @Override
    public boolean ejectSPL(String uuid) throws Exception {
        if (isConnected) {
            String result = playbackControl.ejectSPL();
            deviceLog.trace("ChristieServer ejectSPL, result : {}", result);
            Response response = new Response(result);
            if (response.status() == Response.OK)
                return true;
        }
        return false;
    }

    @Override
    public String createSPL(String s) throws Exception {
        return null;
    }

    public void testChannel(String value) throws Exception {
    	if (isConnected) {
            String result = systemManagement.testChannel(value);
            Response response = new Response(result);
            if (response.status() == Response.OK){
            	
            }
        }
    }

}
