package com.miaozhen.device.support;

/**
 * @Description :
 * Created by liyongbo on 2024-07-16 14:32.
 */

import com.barco.dual.projector.EPlatform;
import com.barco.dual.projector.OSinfo;
import com.miaozhen.device.IDCServer;
import com.miaozhen.device.expection.DeviceException;
import com.miaozhen.device.smi.PlaybackStatus;
import com.miaozhen.device.support.christie.basecmd.UtilTMSChris;
import com.miaozhen.device.support.util.ByteConvertUtil;
import com.miaozhen.device.support.util.IMaxSocketUtil;
import com.miaozhen.util.DateUtil;
import org.apache.commons.codec.binary.Hex;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.*;
import java.net.Socket;
import java.net.SocketTimeoutException;

public class IMaxServer implements IDCServer {

    private final Logger log = LoggerFactory.getLogger(getClass());

//    private String ip = "116.78.116.26";
//    private int port = 11738;
    private String ip ;
    private int port;
    public InputStream is;
    public OutputStream os;

    private final byte packKey1 [] = {6, 14, 43, 52, 2, 5, 1, 10, 14, 16, 1, 1, 1};

    private final byte packKey2 [] = {0};

    public boolean  isConnected = false;
    public Socket socket;

    public IMaxServer(String ip, int port, String userName, String password){
        this.ip = ip;
        this.port = port;
        isConnected = checkConnect();
    }

    public boolean checkConnect() {
        if(!checkNetWorkConnect()){
//            log.debug("IMaxServer is not  connect ip is:{}",ip);
            return false;
        }
        try {
            socket = IMaxSocketUtil.getInstance(ip,port);
            this.is = socket.getInputStream();
            this.os = socket.getOutputStream();
            isConnected = true;
        } catch (SocketTimeoutException e1){
            log.debug("IMaxServer connect time out,please check ip is good");
            return false;
        } catch (IOException e) {
            log.debug("IMaxServer connect IO Exception,please check ip is good is {}",e);
            return false;
        }
        return true;
    }

    public boolean checkNetWorkConnect() {
        EPlatform OSInfo = OSinfo.getOSname();
        Runtime runtime = Runtime.getRuntime();
        Process process = null;
        try {
            if (OSInfo.equals(EPlatform.Windows)) {
                process = runtime.exec("ping " + this.ip + " -n 1 -w 3");

            } else {
                process = runtime.exec("ping " + this.ip + " -c 1 -w 3");

            }
            InputStream is = process.getInputStream();
            InputStreamReader isr = new InputStreamReader(is);
            BufferedReader br = new BufferedReader(isr);
            String line = null;
            StringBuffer sb = new StringBuffer();
            while ((line = br.readLine()) != null) {
                sb.append(line);
            }
            is.close();
            isr.close();
            br.close();
            if (null != sb && !sb.toString().equals("")) {
                if (sb.toString().toLowerCase().indexOf("ttl") > 0) {
                    log.debug("IMaxServer 连接成功{}", this.ip);
                    return true;
                } else {
                    log.debug("IMaxServer 连接失败{}", this.ip);
                    return false;
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error("IMaxServer checkConnect exception : {}", e);
            return false;
        } finally {
            if (process != null) {
                process.destroy();
            }
        }
        return false;
    }

    public static void main(String[] args) throws Exception {
/*        IMaxServer iMaxServer = new IMaxServer("116.78.116.26",11738,"","");
        List<String> list = iMaxServer.getKDMList();

        String uuid = "urn:uuid:39d75d89-b98c-4262-854e-c2c5bbff5acd";
        String xml = iMaxServer.getKDM(uuid);
        System.out.println(xml);
        try {
            KeyDeliveryMessage kdm = KeyDeliveryMessage.create("", xml);
            System.out.println(kdm);
        }catch (Exception e){
            e.printStackTrace();
        }*/
        byte cmdSend[] ={123, 2, 52, 43, 1, 0, 6, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 38, 1, 0, 0, 0, 14, 115, 0, 101, 0, 114, 0, 118, 0, 105, 0, 99, 0, 101, 0, 2, 0, 0, 0, 14, 115, 0, 101, 0, 114, 0, 118, 0, 105, 0, 99, 0, 101, 0};
        String aa = Hex.encodeHexString(cmdSend);
        String bb = ByteConvertUtil.addSpaces(aa,2);
        System.out.println(bb);

        String input = "32363637383100000000000000000000";
        String output = input.replaceAll("0+\\Z", ""); // "\\Z" 匹配字符串结束
        System.out.println(output+"}"); // 输出: 1234
    }

    public static String hexEncode(byte[] input) {
        return Hex.encodeHexString(input);
    }



    public byte[] receive(int length) throws Exception {
        byte[] result = new byte[length];
        int bufsize = (length < this.socket.getReceiveBufferSize()) ? length
                : this.socket.getReceiveBufferSize();
        int size = 0;
        while (size < length) {
            int s = length - size;
            byte[] buffer = new byte[(bufsize > s) ? s : bufsize];
            int readsize = this.is.read(buffer);
            System.arraycopy(buffer, 0, result, size, readsize);
            for (int i = 0; i < result.length; i++) {
//                System.out.print(result[i] + ",");
            }
            size += readsize;
        }
        return result;
    }

    public byte[] receive2(int length,Socket socket) throws Exception {
        byte[] result = new byte[length];
        int bufsize = (length < socket.getReceiveBufferSize()) ? length
                : socket.getReceiveBufferSize();
        int size = 0;
        while (size < length) {
            int s = length - size;
            byte[] buffer = new byte[(bufsize > s) ? s : bufsize];
            int readsize = this.is.read(buffer);
            System.arraycopy(buffer, 0, result, size, readsize);
            for (int i = 0; i < result.length; i++) {
//                System.out.print(result[i] + ",");
            }
            size += readsize;
        }
        return result;
    }
    public byte[] sendAndReceive(byte bytes[], int length) throws Exception {
        send(bytes);
        return receive(length);
    }
    public static int bytes2int(byte[] b) {
//        b=new byte[]{0,0,0,-86};
        int mask=0xff;
        int temp=0;
        int res=0;
        for(int i=0;i<4;i++){
            res<<=8;
            temp=b[i]&mask;
            res|=temp;
        }
        return res;
    }


    public static String handleuuid(String uuid){
        uuid = uuid.replace("-","");
        String header="urn:uuid:";
        if(uuid.startsWith(header)){
            String fin=uuid.substring(9, uuid.length());
            return fin;
        }else{
            return uuid;
        }
    }

    public static String getuuid(String uuid){
        String str1 = uuid.substring(0,8);
        String str2 = uuid.substring(8,12);
        String str3 = uuid.substring(12,16);
        String str4 = uuid.substring(16,20);
        String str5 = uuid.substring(20,uuid.length());
        String str6 = "urn:uuid:"+str1+"-"+str2+"-"+str3+"-"+str4+"-"+str5;
        return str6;
    }

    public void send(byte bytes[]) throws Exception {
        try{
            this.os.write(bytes);
            this.os.flush();
        }catch (Exception e){
            log.debug("IMaxServer send error is",e);
//            if(e.getMessage().contains("close")){
                IMaxSocketUtil.close(ip);
                checkConnect();
                this.os.write(bytes);
                this.os.flush();
//            }
        }
    }

    @Override
    public String getIP() {
        return null;
    }

    @Override
    public void setIP(String ip) {

    }

    @Override
    public String getAuditLog(String ftpIp, String user, String password, String tempLogPath, String startDate, String startTime, String endDate, String endTime) throws Exception {
        log.debug("IMaxServer getAuditLog start");
        if(!isConnected) {
            return null;
        }
        String xml = "";
        try{
            byte getAuditLog[] = {12,1};//        SM LOG
            byte requestId[] = ByteConvertUtil.getRandomUint32();//random
            byte strictMode[] = ByteConvertUtil.uint8ToBytes((byte)0);//0-all logs ,1-SMPTE log record only
            byte filterMask[] = ByteConvertUtil.uint8ToBytes((byte)12);//Filter mask for log record: 0 – no filter 1 – FilterMaskEventIdMin 2 – FilterMaskEventIdMax 4 – FilterMaskUTCTimeMin 8 – FilterMaskUTCTimeMax 16 – FilterMaskCompositionId 32 – FilterMaskKDMId
            byte eventIdMin[] = ByteConvertUtil.uint32ToBytes((byte)0);
            byte eventIdMax[] = ByteConvertUtil.uint32ToBytes((byte)0);
            long timeBeginSeconds = DateUtil.getSecondsSince1970(DateUtil.parse(startDate + " "+ startTime, DateUtil.FORMAT_DATETIMESTAMP));
            long timeEndSeconds = DateUtil.getSecondsSince1970(DateUtil.parse(endDate + " "+ endTime, DateUtil.FORMAT_DATETIMESTAMP));
            byte timeMin[] = ByteConvertUtil.uint64ToBytes(timeBeginSeconds);
            byte timeMax[] = ByteConvertUtil.uint64ToBytes(timeEndSeconds);
            byte cplId[] = Hex.decodeHex("00000000000000000000000000000000".toCharArray());
            byte kdmId[] = Hex.decodeHex("00000000000000000000000000000000".toCharArray());

            int len = requestId.length + strictMode.length + filterMask.length + eventIdMin.length + eventIdMax.length + timeMin.length + timeMax.length + cplId.length + kdmId.length;
            byte[] sendLength = UtilTMSChris.intToBigEndian(len);
            byte[][] cmds = { this.packKey1, getAuditLog, this.packKey2, sendLength, requestId, strictMode, filterMask, eventIdMin, eventIdMax, timeMin, timeMax, cplId, kdmId};
            byte[] cmdSend = UtilTMSChris.byteArrayCopy(cmds);
            String aa = Hex.encodeHexString(cmdSend);
            String bb = ByteConvertUtil.addSpaces(aa,2);
            send(cmdSend);
            receive(16);//pack
            int length = UtilTMSChris.decodeDefiniteFormBER(receive(4));//berLength
            receive(4);//requestID
            byte[] errCode = receive(1);//err code: 0-no error,1-error
            log.debug("IMaxServer getAuditLog errcode : {}",errCode[0]);
            if(length>5){
                byte[] xmlByte = receive(length-5);
                xml = new String(xmlByte,"UTF-8");
                xml = xml.trim();
            }
            log.debug("IMaxServer getAuditLog xml : {}",xml);
        }catch (Exception e){
            log.debug("IMaxServer getAuditLog exception : {}",e);
        }
        return xml;
    }

    @Override
    public PlaybackStatus getPlaybackStatus() throws Exception {
        return null;
    }

    @Override
    public void connect() throws DeviceException {

    }

    @Override
    public boolean ejectSPL(String uuid) throws Exception {
        return false;
    }

    @Override
    public String getServerType() {
        return null;
    }

    @Override
    public void setConnected(boolean connected) {

    }

    @Override
    public boolean getConnected() {
        return false;
    }

    @Override
    public void disconnect() throws DeviceException {

    }
}
