package com.miaozhen.device.support;

import com.barco.dual.projector.EPlatform;
import com.barco.dual.projector.OSinfo;
import com.miaozhen.constants.Constants;
import com.miaozhen.device.IDCServer;
import com.miaozhen.device.expection.DeviceException;
import com.miaozhen.device.response.ledserver.Response;
import com.miaozhen.device.smi.PlaybackStatus;
import com.miaozhen.device.support.util.TrustAllManager;
import com.miaozhen.util.DateUtil;
import com.miaozhen.util.ExpiringCacheUtil;
import com.miaozhen.util.FileUtil;
import com.miaozhen.util.JsonMapper;
import com.pythagorastech.tms.util.PathUtil;
import net.sf.json.JSONArray;
import net.sf.json.JSONObject;
import org.apache.commons.lang.StringUtils;
import org.apache.http.Header;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.HttpClient;
import org.apache.http.client.ResponseHandler;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.methods.HttpDelete;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.config.Registry;
import org.apache.http.config.RegistryBuilder;
import org.apache.http.conn.socket.ConnectionSocketFactory;
import org.apache.http.conn.socket.PlainConnectionSocketFactory;
import org.apache.http.conn.ssl.NoopHostnameVerifier;
import org.apache.http.conn.ssl.SSLConnectionSocketFactory;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.impl.conn.PoolingHttpClientConnectionManager;
import org.apache.http.ssl.SSLContexts;
import org.apache.http.ssl.TrustStrategy;
import org.apache.http.util.EntityUtils;
import org.json.XML;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.net.ssl.*;
import java.io.*;
import java.math.BigDecimal;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLConnection;
import java.security.KeyManagementException;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.text.SimpleDateFormat;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class LedServer implements IDCServer {

    private final Logger logger = LoggerFactory.getLogger(getClass());

    protected String ip = "";

    protected String ledIp;

    protected String ledIpAndPort;

    protected static int port;

    private boolean isLoginSuccess = false;

    private SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");


    private String userName = null;

    private String password = null;

    private Map<String,String> dcpIds = null;

    private final static String ledLogPath = "ledLog";

    private final static int download_wait_time = 5 * 60 * 1000;//每10秒访问一次进度

    private String fpc;
    private String fph;

    public static volatile ConcurrentHashMap<String, Hashtable> cookies = new ConcurrentHashMap<>();

    public ThreadLocal<Integer> getErrorNum = ThreadLocal.withInitial(() -> 0);

    ExpiringCacheUtil<String,Object> cacheUtil = new ExpiringCacheUtil<>(30);//
    ExpiringCacheUtil<String,Object> mediaStateCacheUtil = new ExpiringCacheUtil<>(10);//10秒缓存
    public ExpiringCacheUtil<String,Object> getCacheUtil(){
        return cacheUtil;
    }
    public LedServer(String ip, int port, String userName, String password)
            throws DeviceException {
        logger.debug("LedServer init ip is{}",ip);
        this.userName = userName;
        this.password = password;
        this.ip =  ip;
        this.ledIp = "https://" + ip;
        this.port = port;
        ledIpAndPort = this.ledIp+":"+port;
        if (checkConnect()) {
            connect();
        }else{
            isLoginSuccess = false;
        }
    }

    public boolean checkConnect() {
        EPlatform OSInfo = OSinfo.getOSname();
        Runtime runtime = Runtime.getRuntime();
        Process process = null;
        try {
            if (OSInfo.equals(EPlatform.Windows)) {
                process = runtime.exec("ping " + this.ip + " -n 1 -w 3");

            } else {
                process = runtime.exec("ping " + this.ip + " -c 1 -w 3");

            }
            InputStream is = process.getInputStream();
            InputStreamReader isr = new InputStreamReader(is);
            BufferedReader br = new BufferedReader(isr);
            String line = null;
            StringBuffer sb = new StringBuffer();
            while ((line = br.readLine()) != null) {
                sb.append(line);
            }
            is.close();
            isr.close();
            br.close();
            if (null != sb && !sb.toString().equals("")) {
                logger.debug("LedServer 连接 sb is {} ip is{}", sb.toString() ,this.ip);
                if (sb.toString().toLowerCase().indexOf("ttl") > 0) {
                    logger.debug("LedServer 连接成功{} status is{}", this.ip);
                    return true;
                } else {
                    logger.debug("LedServer 连接失败{}", this.ip);
                    return false;
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            logger.debug("LedServer checkConnect exception : {}", e);
            return false;
        } finally {
            if (process != null) {
                process.destroy();
            }
        }
        return false;
    }
    public HttpClient getInstance(){
        SSLContext sslContext = null;
        try {
            sslContext = SSLContexts.custom()
                    .loadTrustMaterial(new TrustStrategy() {
                        @Override
                        public boolean isTrusted(X509Certificate[] chain, String authType) throws CertificateException {
                            return true;
                        }
                    })
                    .build();
        } catch (NoSuchAlgorithmException e) {
            e.printStackTrace();
        } catch (KeyManagementException e) {
            e.printStackTrace();
        } catch (KeyStoreException e) {
            e.printStackTrace();
        }
        SSLConnectionSocketFactory sslsf = new SSLConnectionSocketFactory(
                sslContext,
                NoopHostnameVerifier.INSTANCE);
        PlainConnectionSocketFactory plainConnectionSocketFactory = new PlainConnectionSocketFactory();
        Registry<ConnectionSocketFactory> registry = RegistryBuilder.<ConnectionSocketFactory>create()
                .register("https", sslsf)
                .register("http", plainConnectionSocketFactory)
                .build();
        PoolingHttpClientConnectionManager connectionManager = new PoolingHttpClientConnectionManager(registry);
        connectionManager.setValidateAfterInactivity(0);//禁用连接池 每次请求都会关闭并重新打开连接

        // 创建请求配置
        RequestConfig requestConfig = RequestConfig.custom()
                .setConnectTimeout(5000) // 连接超时时间5秒e
                .setSocketTimeout(20000)  // 数据读取超时时间10秒r
                .build();

        ResponseHandler<String> handler = new ResponseHandler<String>() {
            @Override
            public String handleResponse(HttpResponse response) throws ClientProtocolException, IOException {
                return EntityUtils.toString(response.getEntity(), "UTF-8");
            }
        };
        return HttpClients.custom()
                .setConnectionManager(connectionManager)
                .setDefaultRequestConfig(requestConfig)
                .build();
    }

    private String getByte(String param){
        Double resD = 0d;
        if(param.contains("T")){
            param = param.replace("T","");
            resD = Double.valueOf(param) * 1000  * 1024 * 1024 * 1024 * 1024 / 1000;
        }else if(param.contains("G")){
            param = param.replace("G","");
            resD = Double.valueOf(param) * 1000 * 1024 * 1024 * 1024 / 1000 ;
        }
        return new BigDecimal(String.valueOf(resD)).toPlainString();
    }

    public static void main(String[] args) {
        String result = "{\"Error\": \"OK\", \"Media\": {\"ATMOSErrorcode\": 0, \"ATMOSMessage\": \"\", \"ATMOSNetworkState\": 0, \"ATMOSState\": 0, \"CPLId\": \"\", \"CPL_Position\": 0, \"CplIndex\": -1, \"Decoder\": \"\", \"Desc\": \"\", \"Error\": 0, \"GdDelay\": 0, \"GdHigh\": 0, \"GdLow\": 0, \"GlobalDimming\": 0, \"Height\": 0, \"PlaybackType\": \"\", \"Position\": \"00:00:00\", \"Resolution\": \"\", \"SPL_Duration\": 0, \"SPL_PlaylistID\": \"\", \"SPL_Position\": 0, \"Spl\": \"\", \"State\": \"Stopped\", \"SyncPlayback\": 0, \"TMSMode\": 1, \"VysncInCount\": 0, \"Width\": 0}, \"Path\": \"/tms/v1/media/control\", \"Status\": 200, \"Timestamp\": \"2025-02-27T04:02:44+00:00\"}\n";
        JSONObject obj = JSONObject.fromObject(result);
        JSONObject media = obj.getJSONObject("Media");
        String state = media.getString("State");
        String playbackType = media.getString("PlaybackType");
    }

    @Override
    public String getIP() {
        return null;
    }

    @Override
    public void setIP(String ip) {

    }

    @Override
    public String getAuditLog(String ftpIp, String user, String password, String tempLogPath, String startDate, String startTime, String endDate, String endTime) throws Exception {
        long time1 = System.currentTimeMillis();
        StringBuffer stringBuffer = new StringBuffer();

        if(isLoginSuccess) {
            String url = ledIpAndPort + "/tms/v1/imb/log/audit";
            Map map = new HashMap();
            String startTime1 = DateUtil.toFormatTimeZone0(startDate + "T" + startTime + "+08:00");
            String endTime1 = DateUtil.toFormatTimeZone0(endDate + "T" + endTime + "+08:00");
            map.put("StartTime",startTime1);
            map.put("EndTime",endTime1);
            StringEntity se = new StringEntity(JsonMapper.getInstance().toJson(map), "application/json","UTF-8");
            String result = post(url, se);
            logger.info("getAuditLog ip is:{} result is:{}",ip,result);
            Response res = JsonMapper.getInstance().fromJsonIgnoreCase(result, Response.class);
            if (res.getStatus() == 200) {

                JSONArray logFile = new JSONArray();
                JSONArray logProcess = new JSONArray();
                while (logFile.size() < 1 || logProcess.size() > 0){// 意味着日志文件没有准备好
                    logger.info("logFilesize is {},logProcess is{}",logFile.size(),logProcess.size());
                    String result2 = get(url);
                    logger.info("getAuditLog result2 is:{}",result2);
                    JSONObject object = JSONObject.fromObject(result2);
                    JSONObject auditLog = object.getJSONObject("AuditLog");
                    logFile = auditLog.getJSONArray("LogFile");
                    logProcess = auditLog.getJSONArray("LogProcess");
                    for (int j = 0; j < logProcess.size(); j++) {
                        JSONObject object1 = (JSONObject) logProcess.get(j);
                        int processProgressPercent = object1.getInt("ProcessProgressPercent");
                        String startTime_ = object1.getString("StartTime");
                        logger.info("ledxxxx get audit log  startTime_ is {},PreocessProgressPercent {}", startTime_, processProgressPercent);
                    }
                    Thread.sleep(download_wait_time);
                }

                String[] fileNames  = new String[logFile.size()];
                for (int i = 0; i < logFile.size(); i++) {
                    String fileName = (String) logFile.get(i);
                    fileNames[i] = fileName;
                    Map map2 = new HashMap();
                    map2.put("FileName",fileName);
                    stringBuffer.append(down(url, JsonMapper.getInstance().toJson(map2), fileName));
                }
                logger.info("download fileNames is{}", fileNames);

            }
        }
        String a = "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\"?>\n" +
                "<LogReport xmlns=\"http://www.smpte-ra.org/schemas/430-4/2008/LogRecord/\" xmlns:dcml=\"http://www.smpte-ra.org/schemas/433/2008/dcmlTypes/\" xmlns:ds=\"http://www.w3.org/2000/09/xmldsig#\" xmlns:lr=\"http://www.smpte-ra.org/schemas/430-4/2008/LogRecord/\" xmlns:xs=\"http://www.w3.org/2001/XMLSchema\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\">\n" +
                "  <lr:reportDate>2023-12-13T09:51:17+00:00</lr:reportDate>\n" +
                "  <lr:reportingDevice>\n" +
                "    <dcml:DeviceIdentifier idtype=\"CertThumbprint\">zCdu8AVKHiSBFni+c+JVcAYOQ/o=</dcml:DeviceIdentifier>\n" +
                "    <dcml:DeviceTypeID scope=\"http://www.smpte-ra.org/schemas/433/2008/dcmlTypes/#device-type-tokens\">PR SM FMI FMA MDI MDA MDS SPB</dcml:DeviceTypeID>\n" +
                "    <dcml:AdditionalID>VteY3920nKs+j8HjAKlQzr3tfu8=</dcml:AdditionalID>\n" +
                "    <dcml:DeviceSerial>000058</dcml:DeviceSerial>\n" +
                "    <dcml:DeviceCertID>zCdu8AVKHiSBFni+c+JVcAYOQ/o=</dcml:DeviceCertID>\n" +
                "    <dcml:ManufacturerName>JoveAI</dcml:ManufacturerName>\n" +
                "    <dcml:ModelNumber>STAR-2000</dcml:ModelNumber>\n" +
                "    <dcml:VersionInfo>\n" +
                "      <dcml:Name>JoveAI IMB SW version</dcml:Name>\n" +
                "      <dcml:Value>1.1.1</dcml:Value>\n" +
                "      <dcml:Name>Log Signer</dcml:Name>\n" +
                "      <dcml:Value>0.2</dcml:Value>\n" +
                "    </dcml:VersionInfo>\n" +
                "  </lr:reportingDevice>";
        String b = "</LogReport>";
        long time2 = System.currentTimeMillis();
        logger.info("ledxxxx getAuditLog time is {}",(time2 - time1)/1000);
        logger.info("LedServer getAuditLog ip is {} is {}", this.ip, stringBuffer.toString());
        return stringBuffer.length() > 0 ? a + stringBuffer.toString() + b : "";
    }

    public String get(String url) throws Exception {
        int code = 0;
        String result = "";
        HttpResponse response = null;
        while(getErrorNum.get() < 2){
            HttpClient httpClient = getInstance();
            HttpGet method = new HttpGet(url);
//            method.setDoAuthentication(true);
//            method.addRequestHeader("Accept","*/*");
//            method.addRequestHeader("Accept-Encoding","gzip, deflate, br");
//            method.addRequestHeader("User-Agent","PostmanRuntime/7.26.8");
//            logger.info("LedServer get is fph is{}",fph);
//            logger.info("LedServer get is fpc is{}",fpc);
            method.addHeader("Cookie","fph=" + fph + "; fpc=" + fpc);
//            method.addRequestHeader("Postman-Token","179249cd-16dd-4693-b57c-f48a6766f0c7");
//            method.addRequestHeader("Connection","keep-alive");
//            method.addRequestHeader("Host","218.241.209.227:8004");

            try {
                response = httpClient.execute(method);
                code = response.getStatusLine().getStatusCode();
                HttpEntity httpEntity = response.getEntity();
                result = EntityUtils.toString(httpEntity,"UTF-8");
//                if(StringUtils.isBlank(result)){
//                    logger.info("LedServer get is null abort");
//                    fph.set("");
//
//                    method.abort();
//                }
                if(code == 200){
                    getErrorNum.set(0);
                    break;
                }else if(code == 401){//登录失效
                    logger.info("LedServer login expire",code);
                    cookies.put(ip,new Hashtable());
                    logout();
                    connect();
                }else{
                    logger.info("LedServer get Status is{}",code);
                }
                getErrorNum.set(getErrorNum.get() + 1);
            } catch (IOException e) {
//                logger.info("LedServer get Exception is{}",e.getMessage());
                if(e.getMessage().contains("Connection refused")){//Led 关闭通道
                    cookies.put(ip,new Hashtable());
                }
            }catch (Exception e) {
                logger.info("LedServer get Exception is{}",e);
            }finally {
                method.releaseConnection();
            }
        }
        if(code!=200){
            logger.info("LedServer get url is {},result is {}  code is:{}", url, result,code);
        }
//        logger.info("LedServer get url is {},code is :{}", url, code);
        if(StringUtils.isBlank(result)){
//            logger.info("LedServer get url is {},response is :{}", url, response);
        }
        return result;
    }

    public String logout() throws Exception {
        String url = ledIpAndPort+"/tms/v1/logout/";
        return get(url);
    }



    private boolean delete(String url){
        int code = 0;
        String result = "";
        HttpResponse response = null;
        HttpDelete method = null;
        int errnum = 0;
        // StringBuffer stringBuffer = new StringBuffer();

        while(errnum < 2) {
            try {
                HttpClient httpClient = getInstance();
                method = new HttpDelete(url);
                method.addHeader("Cookie", "fph=" + fph + "; fpc=" + fpc);
                response = httpClient.execute(method);
                code = response.getStatusLine().getStatusCode();
                HttpEntity httpEntity = response.getEntity();
                result = EntityUtils.toString(httpEntity, "UTF-8");

                if (code == 200) {
                    break;
                    //                logger.info("LedExecute delete url is {},response is {}", url, response);
                } else if (code == 401) {
                    //                logger.info("LedExecute delete url is {},ccokie is expire, response is {}", url, response);
                    logger.info("LedServer login expire", code);
                    cookies.put(ip,new Hashtable());
                    logout();
                    connect();
                } else {
                    logger.info("LedServer delete url is {},result is {}", url, result.toString());
                }
                logger.info("LedExecute delete url is {},code is:{},response is {}", url, code, response);
            } catch (Exception e) {
                logger.info("LedServer delete Exception is", e);
                cookies.put(ip,new Hashtable());
            } finally {
                errnum++;
                method.releaseConnection();
            }
        }
        Response res = JsonMapper.getInstance().fromJsonIgnoreCase(result,Response.class);
        return res.getStatus()==200 ? true : false;
    }

    public String down(String urlStr, String params, String fileName) throws IOException, KeyManagementException, NoSuchAlgorithmException {
        StringBuffer sb = new StringBuffer();
        HttpURLConnection conn = null;
        try{
            URL url = new URL(urlStr);
            conn = (HttpURLConnection) url.openConnection();
            trustAllHttpsCertificates(conn);//SSL
            conn.setRequestMethod("POST");
            // 设置超时间为3秒
            conn.setConnectTimeout(3 * 1000);
            conn.setDoOutput(true);
            conn.setDoInput(true);
            // 防止屏蔽程序抓取而返回403错误
            conn.setRequestProperty("Cookie","fph=" + fph + "; fpc=" + fpc);
            conn.setRequestProperty("Content-Length", params.getBytes().length + "");
            conn.setRequestProperty("Content-Type", "application/json");

            OutputStream outputStream = conn.getOutputStream();
            outputStream.write(params.getBytes());
            outputStream.flush();
            outputStream.close();

            // 得到输入流
            InputStream inputStream = conn.getInputStream();
            // 获取自己数组
            byte[] getData = readInputStream(inputStream);
            // 文件保存位置
            String u = PathUtil.webappPath() + File.separator + Constants.TEMPLOG + File.separator + ledLogPath;
//            String u = "D:\\log\\";
            File saveDir = new File(u);
            if (!saveDir.exists()) {
                saveDir.mkdir();
            }
            File file = new File(saveDir + File.separator + fileName);
            FileOutputStream fos = new FileOutputStream(file);
            fos.write(getData);
            if (fos != null) {
                fos.close();
            }
            if (inputStream != null) {
                inputStream.close();
                conn.disconnect();
            }
            logger.info("download fileName is {}", fileName);
            String destPath = u + File.separator + fileName.split("\\.")[0];
            File destDir = new File(destPath);
            FileUtil.zipDecompress(file, destDir);
            File[] files = destDir.listFiles();
            logger.info("download files is {}",files);
            for (int i = 0; i < files.length; i++) {
                String xml = file2XML(files[i]);
                sb.append(xml);
            }
        }catch (Exception e){
            logger.info("download error is{}",e);
        }
        return sb.toString();
    }

    public static byte[] readInputStream(InputStream inputStream) throws IOException {
        byte[] buffer = new byte[1024];
        int len = 0;
        ByteArrayOutputStream bos = new ByteArrayOutputStream();
        while ((len = inputStream.read(buffer)) != -1) {
            bos.write(buffer, 0, len);
        }
        bos.close();
        return bos.toByteArray();
    }

    // 针对单个连接
    public static void trustAllHttpsCertificates(URLConnection connection) throws NoSuchAlgorithmException, KeyManagementException {
        HttpsURLConnection httpsURLConnection = (HttpsURLConnection) connection;
        TrustManager[] trustAllCerts = new TrustManager[1];
        trustAllCerts[0] = new TrustAllManager();
        SSLContext sc = SSLContext.getInstance("SSL");
        sc.init(null, trustAllCerts, null);
        httpsURLConnection.setSSLSocketFactory(sc.getSocketFactory());
        httpsURLConnection.setHostnameVerifier(new HostnameVerifier() {
            @Override
            public boolean verify(String urlHostName, SSLSession session) {
                return true;
            }
        });
    }

    private static String file2XML(File file) throws IOException {
        String xml = FileUtil.readFileToString(file);
        org.json.JSONObject logReport = XML.toJSONObject(xml).getJSONObject("LogReport");
        org.json.JSONArray record = logReport.getJSONArray("lr:LogRecordElement");
        return XML.toString(record, "lr:LogRecordElement");
    }

    public String post(String url, HttpEntity he) throws Exception {
        int code = 0;
        String result = "";
        int errnum = 0;
        HttpResponse response = null;
        HttpPost method = null;
        while(errnum < 2){
            HttpClient httpClient = getInstance();
            method = new HttpPost(url);
            if(he != null){
                method.setEntity(he);
                method.setHeader("Content-Type","application/json");
            }
//          method.setDoAuthentication(true);
            method.addHeader("Cookie","fph=" + fph + "; fpc=" + fpc);
            try {
                response = httpClient.execute(method);
                code = response.getStatusLine().getStatusCode();
                HttpEntity httpEntity = response.getEntity();
                result = EntityUtils.toString(httpEntity,"UTF-8").trim();
                if(code == 200){
                    break;
                }else if(code == 401){//登录失效
                    logger.info("LedServer post cookie expire");
                    cookies.put(ip,new Hashtable());
                    logout();
                    connect();
                }else if(code == 409){
                    break;
                }else{
                    logger.info("LedServer post Status is: {}",code);
                }
            } catch (IOException e) {
                logger.info("LedServer post Exception is ",e);
//               cookies.put(ip,new Hashtable());
            } finally {
                errnum ++;
                method.releaseConnection();
            }
        }
        if(code!=200){
            logger.info("LedServer post url is {},result is {}  code is:{}", url, result,code);
        }
        logger.info("LedServer post url is {},code is:{}", url,code);
        if(StringUtils.isBlank(result)){
            logger.info("LedServer post url is {},response is {}", url, response);
        }
        return result;
    }

    @Override
    public PlaybackStatus getPlaybackStatus() throws Exception {
        return null;
    }

    @Override
    public void connect() throws DeviceException {
        if(!cookies.containsKey(ip) || cookies.get(ip).isEmpty()){
            String url = ledIpAndPort+"/tms/v1/login";
            if (StringUtils.isBlank(userName)) {
                userName = "Maintenance";
            }
            if (StringUtils.isBlank(password)) {
                password = "96694201";
            }
            Map map = new HashMap();
            map.put("Level","0");
            map.put("User",userName);
            map.put("Password",password);
            try {
                StringEntity se = new StringEntity(JsonMapper.getInstance().toJson(map), "application/json","UTF-8");
                getCooike(url, se);
                logger.info("LedServer request this ip {} cookie is {}", this.ip, fpc + fph);
            } catch (UnsupportedEncodingException e) {
                e.printStackTrace();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }else{
            logger.debug("LedServer connect ip cookies is{}",JsonMapper.getInstance().toJson(cookies));
            fpc = (String) cookies.get(ip).get("fpc");
            fph = (String) cookies.get(ip).get("fph");
        }
    }

    public String getCooike(String url, HttpEntity he){
        int code = 0;
        String result = StringUtils.EMPTY;
        try {
            HttpClient client = getInstance();
            HttpPost httpPost = new HttpPost(url);
            httpPost.setEntity(he);
            httpPost.setHeader("Content-Type","application/json");
            HttpResponse response = client.execute(httpPost);
            code = response.getStatusLine().getStatusCode();
            if(code==200){
                HttpEntity httpEntity = response.getEntity();
                Header[] headers = response.getHeaders("Set-Cookie");
                String a = String.valueOf(headers[0]);
                String b = String.valueOf(headers[1]);
//                logger.info("LedServer fpc text is{}",a);
//                logger.info("LedServer fph text is{}",b);
                Pattern pattern = Pattern.compile("fpc=.*?;");
                Matcher matcher = pattern.matcher(a);
                Pattern pattern1 = Pattern.compile("fph=.*?;");
                Matcher matcher1 = pattern1.matcher(b);
                if (matcher.find()){
                    fpc=matcher.group().replace("fpc=","").replace(";","");
                }
                if (matcher1.find()){
                    fph=matcher1.group().replace("fph=","").replace(";","");
                }
                String ip = this.ip;
                Map map = new HashMap();
                map.put("Cookie","fpc=" + fpc + "; " + "fph=" + fph);
                isLoginSuccess = true;
                Hashtable cookie = new Hashtable();
                cookie.put("fpc",fpc);
                cookie.put("fph",fph);
                cookies.put(ip,cookie);
            }else{
                isLoginSuccess = false;
                logger.info("LedServer getCooike url is {},code is {}",url,code);
            }
        } catch (IOException e) {
            isLoginSuccess = false;
        }

        return result;
    }

    @Override
    public boolean ejectSPL(String uuid) throws Exception {
        return false;
    }

    @Override
    public String getServerType() {
        return null;
    }

    @Override
    public void setConnected(boolean connected) {

    }

    @Override
    public boolean getConnected() {
        return false;
    }

    @Override
    public void disconnect() throws DeviceException {

    }
}
