package com.miaozhen.device.support;

import com.barco.dual.projector.EPlatform;
import com.barco.dual.projector.OSinfo;
import com.miaozhen.util.ExpiringCacheUtil;
import com.pythagorastech.tms.device.dcserver.IDCServer;
import com.pythagorastech.tms.device.dcserver.smi.*;
import com.pythagorastech.tms.device.dcserver.util.MySecureProtocolSocketFactory;
import com.pythagorastech.tms.device.expection.DeviceException;
import net.sf.json.JSONArray;
import net.sf.json.JSONObject;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.MultiThreadedHttpConnectionManager;
import org.apache.commons.httpclient.methods.*;
import org.apache.commons.httpclient.params.HttpConnectionManagerParams;
import org.apache.commons.httpclient.protocol.Protocol;
import org.apache.commons.httpclient.protocol.ProtocolSocketFactory;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.*;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.ZonedDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;

public class QubeServer implements IDCServer {

    private final Logger logger = LoggerFactory.getLogger(getClass());

    protected String ip;

    protected static int port;

    protected HttpClient httpClient = null;

    public static MultiThreadedHttpConnectionManager connectionManager = null;

    private boolean isLoginSuccess = false;
    boolean isConnected = false;

    private SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");

    private String userName = null;

    private String password = null;

    private String serverIP;

    ExpiringCacheUtil<String,Object> cacheUtil = new ExpiringCacheUtil<>(20);

    public QubeServer(String ip, int port, String userName, String password)
            throws DeviceException {
        this.serverIP = ip;
        this.userName = userName;
        this.password = password;
        this.ip = "http://" + ip;
        this.port = 80;
        if(checkConnect()){
            getInstance();
        }
    }

    public boolean checkConnect() {
        EPlatform OSInfo = OSinfo.getOSname();
        Runtime runtime = Runtime.getRuntime();
        Process process = null;
        try {
            if (OSInfo.equals(EPlatform.Windows)) {
                process = runtime.exec("ping " + this.serverIP + " -n 1 -w 3");

            } else {
                process = runtime.exec("ping " + this.serverIP + " -c 1 -w 3");

            }
            InputStream is = process.getInputStream();
            InputStreamReader isr = new InputStreamReader(is);
            BufferedReader br = new BufferedReader(isr);
            String line = null;
            StringBuffer sb = new StringBuffer();
            while ((line = br.readLine()) != null) {
                sb.append(line);
            }
            is.close();
            isr.close();
            br.close();
            if (null != sb && !sb.toString().equals("")) {
                if (sb.toString().toLowerCase().indexOf("ttl") > 0) {
                    logger.debug("QubeServer 连接成功{}", this.serverIP);
                    isConnected = true;
                    return true;
                } else {
                    logger.debug("QubeServer 连接失败{}", this.serverIP);
                    return false;
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("QubeServer checkConnect exception : {}", e);
            return false;
        } finally {
            if (process != null) {
                process.destroy();
            }
        }
        return false;
    }

    @Override
    public String getCPL(String uuid) throws Exception {
        if(!isConnected) {
            return null;
        }
        if(cacheUtil.get("cpl_"+uuid)!=null){
            return (String)cacheUtil.get("cpl_"+uuid);
        }else{
            String url = ip + "/api/v1/media/cpls/" + removePrefix(uuid) + "/xml";
            String result = get(url);
//        logger.debug("QubeServer getCPL result is:{}",result);
            return result;
        }
    }

    @Override
    public List<String> getCPLList() throws Exception {
        return null;
    }

    @Override
    public boolean deleteContent(String uuid) throws Exception {
        if(!isConnected) {
            return false;
        }
        String url = ip + "/api/v1/media/cpls/" + removePrefix(uuid);
        String result = delete(url);
        return true;
    }

    @Override
    public boolean validateCPL(String s) throws Exception {
        return false;
    }

    @Override
    public String getKDM(String uuid) throws Exception {
        if(!isConnected) {
            return null;
        }
        String url = ip + "/api/v1/media/kdms/" + removePrefix(uuid) + "/xml";
        String result = get(url);
        return result;
    }

    @Override
    public List<String> getKDMList() throws Exception {
        return null;
    }

    @Override
    public boolean deleteKDM(String uuid) throws Exception {
        if(!isConnected) {
            return false;
        }
        String url = ip + "/api/v1/media/kdms?id=" + removePrefix(uuid);
        String result = delete(url);
        logger.debug("QubeServer deleteKDM result is:{}",result);
        return true;
    }

    @Override
    public PlaybackStatus getPlaybackStatus() throws Exception {
        return null;
    }

    @Override
    public boolean playSPL() throws Exception {
        if(!isConnected) {
            return false;
        }
        String url = ip+"/api/v1/playback/play";
        String result = post(url);
        logger.debug("Qube playSPL result is:{}",result);
        return true;
    }

    @Override
    public boolean pauseSPL() throws Exception {
        if(!isConnected) {
            return false;
        }
        String url = ip+"/api/v1/playback/pause";
        String result = post(url);
        logger.debug("Qube pauseSPL result is:{}",result);
        return true;
    }

    @Override
    public boolean stopSPL() throws Exception {
        if(!isConnected) {
            return false;
        }
        String url = ip+"/api/v1/playback/stop";
        String result = post(url);
        logger.debug("Qube stopSPL result is:{}",result);
        return true;
    }

    @Override
    public boolean next() throws Exception {
        return false;
    }

    @Override
    public boolean previous() throws Exception {
        if(!isConnected) {
            return false;
        }
        String url = ip+"/api/v1/playback/eject";
        String result = post(url);
        logger.debug("Qube previous result is:{}",result);
        return true;
    }

    @Override
    public boolean gotoPosition(int offset) throws Exception {
        return false;
    }

    @Override
    public boolean selectSPL(String uuid) throws Exception {
        return false;
    }

    @Override
    public boolean ejectSPL(String uuid) throws Exception {
        if(!isConnected) {
            return false;
        }
        String url = ip+"/api/v1/playback/eject";
        String result = post(url);
        logger.debug("Qube ejectSPL result is:{}",result);
        return true;
    }

    @Override
    public String createSPL(String spl) throws Exception {
        return "false";
    }

    @Override
    public List<String> getSPLList() throws Exception {
        if(!isConnected) {
            return null;
        }
        if(cacheUtil.get("splList_")!=null){
            return (List<String>)cacheUtil.get("splList_");
        }else{
            String url = ip+"/api/v1/media/spls";
            String result = get(url);
            logger.debug("getSPLList result is:{}",result);
            if(StringUtils.isNotBlank(result)){
                List<String> splList = new ArrayList<>();
                JSONObject jsonObject = JSONObject.fromObject(result);
                JSONArray splArr = jsonObject.getJSONArray("spls");
                if(splArr!=null&&splArr.size()>0) {
                    for (int i = 0; i < splArr.size(); i++) {
                        JSONObject obj = (JSONObject) splArr.get(i);
                        String id = obj.getString("id");
                        String splUUID = getUrnUUID(id);
                        splList.add(splUUID);
                    }
                    if(splList!=null&&splList.size()>0){
                        return splList;
                    }
                }
            }
        }
        return null;
    }

    @Override
    public String getSPL(String uuid) throws Exception {
        if(!isConnected) {
            return null;
        }
        if(cacheUtil.get("spl_"+uuid)!=null){
            return (String)cacheUtil.get("spl_"+uuid);
        }else{
            String url = ip+"/api/v1/media/spls/"+uuid+"/xml";
            String result = get(url);
            logger.debug("getSPL result is:{}",result);
            if(StringUtils.isNotBlank(result)){
                return result;
            }
        }
        return null;
    }

    @Override
    public boolean validateSPL(String s) throws Exception {
        return false;
    }

    @Override
    public boolean deleteSPL(String uuid) throws Exception {
        if(!isConnected) {
            return false;
        }
        String url = ip+"/api/v1/media/spls/"+removePrefix(uuid);
        String result = delete(url);
        logger.debug("deleteSPL url is:{} result is:{}",url,result);
        if(StringUtils.isNotBlank(result)){
            if(result.contains("httpStatus")){
                return false;
            }
        }else{
            return true;
        }
        return false;
    }

    @Override
    public String createSchedule(String s, String s1) throws Exception {
        return null;
    }

    @Override
    public List<Schedule> getScheduleList(String scheduleDate) throws Exception {
//      {"showId": "96f5247a-236f-4cc8-a582-e97c804913b9","showName": "误判","showTime": "2025-01-26T19:25:00+08:00","durationInMilliSecs": 7233458}
        if(!isConnected) {
            return null;
        }
        String url = ip+"/api/v1/schedules";
        String result = get(url);
        List<Schedule> scheduleList = new ArrayList<>();
        if(StringUtils.isNotBlank(result)){
            JSONObject jsonObject = JSONObject.fromObject(result);
            JSONArray scheduleArr = jsonObject.getJSONArray("schedules");
            if(scheduleArr!=null&&scheduleArr.size()>0){
               for(int i=0;i<scheduleArr.size();i++){
                   JSONObject obj = (JSONObject)scheduleArr.get(i);
                   String showId = obj.getString("showId");//splUUID
                   String showTime = obj.getString("showTime").split("\\+")[0];
                   String scheduleUUID = getUrnUUID(showId);
                   String showUUID = getUrnUUID(showId);
                   Schedule schedule = new Schedule(scheduleUUID,showUUID, showTime);
                   scheduleList.add(schedule);
               }
            }
        }
        return scheduleList;
    }

    @Override
    public Schedule getSchedule(String uuid) throws Exception {
        return null;
    }

    @Override
    public boolean cancelSchedule(Schedule schedule) throws Exception {
        if(!isConnected) {
            return false;
        }
        String startTime = schedule.getISODateTime()+"+08:00";
        String url = ip+"/api/v1/schedules/"+startTime;
        logger.debug("cancelSchedule schuuid is:{},iosDateTime is:{},startTime is:{}",schedule.getUUID(),schedule.getISODateTime(),startTime);
        logger.debug("cancelSchedule url is:{}",url);
        String result = delete(url);
        logger.debug("cancelSchedule result is:{}",result);
        if(StringUtils.isBlank(result)){
            return true;
        }
        return false;
    }

    @Override
    public boolean enableScheduler() throws Exception {
        return false;
    }

    @Override
    public boolean disableScheduler() throws Exception {
        return false;
    }

    @Override
    public boolean getSchedulerStatus() throws Exception {
        return false;
    }

    @Override
    public String getServerDateTime() throws Exception {
//        {"dateTime":"2025-02-19T11:22:06+08:00","timeZone":"Asia/Hong_Kong","firstDayOfWeek":"Monday","isAutoNtpSyncEnabled":false,"ntpServerAddress":"time.nist.gov"}
        if(!isConnected) {
            return null;
        }
        if(cacheUtil.get("serverDateTime")==null){
            String url = ip+"/api/v1/system/time";
            String result = get(url);
            JSONObject jsonObject = JSONObject.fromObject(result);
            String dateTime = jsonObject.getString("dateTime");
            logger.debug("Qube getServerDateTime result is:{}",result);
            return dateTime;
        }else{
            return (String)cacheUtil.get("serverDateTime");
        }
    }

    @Override
    public ServerInfo getServerInfo() throws Exception {
        return null;
    }

    @Override
    public StorageInfo getStorageInfo() throws Exception {
        if(!isConnected) {
            return null;
        }
        String url = ip+"/api/v1/system/storage/status";
        String result = get(url);
//        logger.debug("getStorageInfo result is:{}",result);
        if(StringUtils.isNotBlank(result)){
            JSONObject jsonObject = JSONObject.fromObject(result);
            String capacityInBytes = jsonObject.getString("capacityInBytes");//总容量
            String occupiedInBytes = jsonObject.getString("occupiedInBytes");//已用容量
//            logger.debug("getStorageInfo capacityInBytes is:{},occupiedInBytes is:{}",capacityInBytes,occupiedInBytes);
            long freeSpace = Long.parseLong(capacityInBytes)-Long.parseLong(occupiedInBytes);
            StorageInfo storageInfo = new StorageInfo();
            storageInfo.setFreeSpace(occupiedInBytes);
            storageInfo.setTotalSpace(String.valueOf(freeSpace));
            return storageInfo;
        }
        return null;
    }

    @Override
    public String getPublickeyThumbprint() throws Exception {
        return null;
    }

    @Override
    public String ingestPackage(String host, String port, String user, String password, String cpl_UUID, String cpl_path) throws Exception {
        logger.debug("QubeServer ingestPackage begin");
        try {
            return null;
       /*     String postUrl = ip + "/api/v1/ingests";
            String dir = cpl_path.substring(0, cpl_path.lastIndexOf("/"));
            String assetMapPath = PathUtil.getAssetMapPath(host, user, password, port, dir);

            IngestRequestBody ingestRequestBody = new IngestRequestBody();
            IngestableEntity ingestableEntity = new IngestableEntity();
            // String url = "ftp://" + user + ":" + password + "@" + host + ":" + port + "/" + cpl_path;
            String url = "ftp://" + user + ":" + password + "@" + host + ":" + port + "/" + dir + "/" + assetMapPath;
            Cpl cpl = new Cpl(cpl_UUID, cpl_UUID);
            ingestableEntity.setUrl(url);
            ingestableEntity.setName(cpl_UUID);
            ingestableEntity.setType("dcp");
            ingestableEntity.setId(cpl_UUID);
            ingestableEntity.setCpl(cpl);
            ingestRequestBody.setIngestableEntity(ingestableEntity);
            ingestRequestBody.setIngestInPlace(true);

            String result = postJson(postUrl, JsonMapper.getInstance().toJson(ingestRequestBody));
            logger.debug("QubeServer ingestPackage result is:{}", result);*/

          //  result is:{"httpStatus":500,"httpCode":"internal_server_error","requestId":"","errors":[]}



        }catch (Exception e){
            logger.debug("QubeServer ingestPackage Exception", e);
        }

        return null;
    }

    @Override
    public String ingestKDM(String host, String port, String user, String password, String path, String uuid) throws Exception {
        return "true";
    }

    @Override
    public IngestStatus getIngestStatus(String uuid, String importPath) throws Exception {
        return null;
    }

    @Override
    public boolean cancelIngest(String uuid) throws Exception {
        return false;
    }

    @Override
    public String getAuditLog(String ftpIp, String user, String password, String tempLogPath, String startDate, String startTime, String endDate, String endTime) throws Exception {
        if(!isConnected) {
            return null;
        }
        String beijingStartTime = startDate + " " + startTime;
        String beijingEndTime = endDate + " " + endTime;
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        ZonedDateTime utcZonedDateStartTime = LocalDateTime.parse(beijingStartTime, formatter).atZone(ZoneId.of("Asia/Shanghai")).withZoneSameInstant(ZoneId.of("UTC"));
        ZonedDateTime utcZonedDateEndTime = LocalDateTime.parse(beijingEndTime, formatter).atZone(ZoneId.of("Asia/Shanghai")).withZoneSameInstant(ZoneId.of("UTC"));

        String url = ip+"/api/v1/logs/security?" +
                "start-timestamp="+utcZonedDateStartTime.format(formatter).replaceAll(" ","T").replaceAll(":","%3A")+".000Z&" +
                "" +
                "end-timestamp="+utcZonedDateEndTime.format(formatter).replaceAll(" ","T").replaceAll(":","%3A")+".000Z";
        logger.debug("Qube getAuditLog url is:{}",url);
        connectionManager.getParams().setSoTimeout(5 * 60 * 1000);
        httpClient.getHttpConnectionManager().getParams().setSoTimeout(5 * 60 * 1000);
        String result = get(url);
        connectionManager.getParams().setSoTimeout(10000);
        httpClient.getHttpConnectionManager().getParams().setSoTimeout(3000);
        logger.debug("Qube getAuditLog result is:{}",result);
        return result;
    }

    @Override
    public List<ServerLogEntity> parseAuditLog(String xml) throws Exception {
        return null;
    }

    @Override
    public List<Cue> getCueList() throws Exception {
        if(!isConnected) {
            return null;
        }
        if(cacheUtil.get("cueList_")!=null){
            return (List<Cue>)cacheUtil.get("cueList_");
        }else{
            String url = ip+"/api/v1/automation/cues";
            String result = get(url);
            JSONObject jsonObject = JSONObject.fromObject(result);
            JSONArray cues = jsonObject.getJSONArray("cues");
            List<Cue> cueList = new ArrayList<>();
            for (int i = 0; i < cues.size(); i++) {
                JSONObject cue = cues.getJSONObject(i);
                String name = cue.getString("name");
                Cue cue1 = new Cue();
                cue1.setCueName(name);
//            cue1.setCueUUID(uuid); ?
                cueList.add(cue1);
            }
            logger.debug("Qube getCueList result is:{}",result);
            return cueList;
        }
    }

    @Override
    public boolean executeCue(String cueLable) throws Exception {
        return false;
    }

    @Override
    public void setConnected(boolean connected) {
        isConnected = connected;
    }

    public HttpClient getInstance() {
        if (connectionManager == null) {
            int maxThreadsTotal = 200;
            int maxThreadsPerHost = 50;
            connectionManager = new MultiThreadedHttpConnectionManager();
            HttpConnectionManagerParams params = connectionManager.getParams();
            params.setMaxTotalConnections(maxThreadsTotal);
            if (maxThreadsTotal > maxThreadsPerHost) {
                params.setDefaultMaxConnectionsPerHost(maxThreadsPerHost);
            } else {
                params.setDefaultMaxConnectionsPerHost(maxThreadsTotal);
            }
            connectionManager.setParams(params);
            connectionManager.closeIdleConnections(0);
            connectionManager.getParams().setSoTimeout(10000);
            connectionManager.getParams().setConnectionTimeout(5000);
        }
        if (httpClient == null) {
            // connectionManager.setMaxTotalConnections (5);//总的连接数
            // connectionManager.setMaxConnectionsPerHost (10);//每个host的最大连接数
            httpClient = new HttpClient(connectionManager);
            ProtocolSocketFactory fcty = new MySecureProtocolSocketFactory();
            Protocol.registerProtocol("https", new Protocol("https", fcty, port));
            // httpClient.getHttpConnectionManager().getParams()
            // .setConnectionTimeout(3000);
            httpClient.getParams().setConnectionManagerTimeout(10000);
            httpClient.getHttpConnectionManager().getParams().setSoTimeout(3000);
            httpClient.getHttpConnectionManager().getParams()
                    .setConnectionTimeout(5000);// 由于要上传的文件可能比较大
            // ,
            // 因此在此设置最大的连接超时时间
        }
        return httpClient;
    }

    @Override
    public String getIP() {
        return ip;
    }

    @Override
    public void setIP(String ip) {
        this.ip = ip;
    }

    @Override
    public void connect() throws DeviceException {

    }

    @Override
    public void disconnect() throws DeviceException {

    }
    public String get(String url) throws IOException {
        String result = "";
        GetMethod method = new GetMethod(url);
        try{
            method.setDoAuthentication(true);
            method.addRequestHeader("Authorization", "Basic YWRtaW46YWRtaW5pc3RyYXRvcg==");
            httpClient.executeMethod(method);
         //   logger.debug("Qube get");
            InputStream inputStream = method.getResponseBodyAsStream();
            BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(inputStream,"UTF-8"));
            StringBuffer stringBuffer = new StringBuffer();
            String str = StringUtils.EMPTY;
            while ((str = bufferedReader.readLine()) != null) {
                stringBuffer.append(str);
            }
            result = stringBuffer.toString();
        }catch (Exception e){
            e.printStackTrace();
        }finally {
            method.releaseConnection();
        }
        return result;
    }

    public String post(String url) {
        String result = "";
        PostMethod method = new PostMethod(url);
        try{
            method.setDoAuthentication(true);
            method.addRequestHeader("Authorization", "Basic YWRtaW46YWRtaW5pc3RyYXRvcg==");
            httpClient.executeMethod(method);
            InputStream inputStream = method.getResponseBodyAsStream();
            BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(inputStream,"UTF-8"));
            StringBuffer stringBuffer = new StringBuffer();
            String str = StringUtils.EMPTY;
            while ((str = bufferedReader.readLine()) != null) {
                stringBuffer.append(str);
            }
            result = stringBuffer.toString();
        }catch (Exception e){
          e.printStackTrace();
        }finally {
            method.releaseConnection();
        }
        return result;
    }
    public int postWithRequestEntity(String url,String content) {
        int result = 0;
        String resultStr = "";
        PostMethod postMethod = new PostMethod(url);
        try{
            postMethod.setDoAuthentication(true);
            postMethod.addRequestHeader("Authorization", "Basic YWRtaW46YWRtaW5pc3RyYXRvcg==");
            StringRequestEntity stringRequestEntity = new StringRequestEntity(
                    new String(content.getBytes("UTF-8")), "application/xml", "UTF-8");
            postMethod.setRequestEntity(stringRequestEntity);
            result = httpClient.executeMethod(postMethod);
            logger.debug("postWithRequestEntity url is:{} result is:{}",url,result);
            InputStream inputStream = postMethod.getResponseBodyAsStream();
            BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(inputStream,"UTF-8"));
            StringBuffer stringBuffer = new StringBuffer();
            String str = StringUtils.EMPTY;
            while ((str = bufferedReader.readLine()) != null) {
                stringBuffer.append(str);
            }
            resultStr = stringBuffer.toString();
            logger.debug("postWithRequestEntity resultStr is:{}",resultStr);
        }catch (Exception e){
            e.printStackTrace();
        }finally {
            postMethod.releaseConnection();
        }
       return result;
    }
    public String postJson(String url,String jsonData) {
        String result = "";
        PostMethod method = new PostMethod(url);
        try {
            method.setDoAuthentication(true);
            method.addRequestHeader("Authorization", "Basic YWRtaW46YWRtaW5pc3RyYXRvcg==");
            method.addRequestHeader("Content-Type","application/json;charset=utf-8");
            method.setRequestEntity(new ByteArrayRequestEntity(jsonData.getBytes("utf-8")));
            httpClient.executeMethod(method);
            InputStream inputStream = method.getResponseBodyAsStream();
            BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(inputStream,"UTF-8"));
            StringBuffer stringBuffer = new StringBuffer();
            String str = StringUtils.EMPTY;
            while ((str = bufferedReader.readLine()) != null) {
                stringBuffer.append(str);
            }
            result = stringBuffer.toString();
        }catch (Exception e){
            e.printStackTrace();
        }finally {
            method.releaseConnection();
        }
        return result;
    }
    public String delete(String url) throws Exception {
        String result = "";
        int code = 0;
        DeleteMethod method = new DeleteMethod(url);
        try{
            method.setDoAuthentication(true);
            method.addRequestHeader("Authorization", "Basic YWRtaW46YWRtaW5pc3RyYXRvcg==");
            code = httpClient.executeMethod(method);
            InputStream inputStream = method.getResponseBodyAsStream();
            BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(inputStream,"UTF-8"));
            StringBuffer stringBuffer = new StringBuffer();
            String str = StringUtils.EMPTY;
            while ((str = bufferedReader.readLine()) != null) {
                stringBuffer.append(str);
            }
            result = stringBuffer.toString();
            logger.debug("delete result is:{},code is:{}",result,code);
        }catch (Exception e){
            logger.info("QubeServer delete error url:{}",url,e);
        }finally {
            method.releaseConnection();
        }
        return result;
    }

    /**
     * uuid添加前缀：urn:uuid:
     * @param uuid
     * @return
     */
    public String getUrnUUID(String uuid){
        String result = "";
        if(!uuid.contains("urn:uuid")){
            result = "urn:uuid:"+uuid;
        }
        return result;
    }

    public String removePrefix(String uuid){
        String header="urn:uuid:";
        if(uuid.startsWith(header)){
            String fin=uuid.substring(9, uuid.length());
            return fin;
        }else{
            return uuid;
        }
    }


    public static void main(String[] args) throws Exception{
    }



}
