package com.miaozhen.device.support.util;

import java.math.BigInteger;
import java.net.InetAddress;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.charset.StandardCharsets;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.Date;
import java.util.Random;
import java.util.TimeZone;

public class ByteConvertUtil {

    public static byte toUint8(byte[] bytes) {
        return bytes[0];
    }
    public static short toUint16(byte[] bytes) {
        int offset = 0; // 如果是从数组的非零位置读取，请更改offset
        return (short) ((bytes[offset] & 0xFF) << 8 | (bytes[offset + 1] & 0xFF));
    }

    public static int toUint32(byte[] bytes) {
        int offset = 0; // 如果是从数组的非零位置读取，请更改offset
        return (bytes[offset] & 0xFF) << 24 |
                (bytes[offset + 1] & 0xFF) << 16 |
                (bytes[offset + 2] & 0xFF) << 8 |
                (bytes[offset + 3] & 0xFF);
    }

    public static long toUint64(byte[] bytes) {
        int offset = 0; // 如果是从数组的非零位置读取，请更改offset
        return ((long) bytes[offset] & 0xFF) << 56 |
                ((long) bytes[offset + 1] & 0xFF) << 48 |
                ((long) bytes[offset + 2] & 0xFF) << 40 |
                ((long) bytes[offset + 3] & 0xFF) << 32 |
                ((long) bytes[offset + 4] & 0xFF) << 24 |
                ((long) bytes[offset + 5] & 0xFF) << 16 |
                ((long) bytes[offset + 6] & 0xFF) << 8 |
                ((long) bytes[offset + 7] & 0xFF);
    }
    public static long bytesToUInt64LE(byte[] bytes) {
        if (bytes == null || bytes.length != 8) {
            throw new IllegalArgumentException("Byte array must be non-null and exactly 8 bytes long.");
        }
        // 反转字节数组，将小端转换为大端
        byte[] reversed = new byte[8];
        for (int i = 0, j = 7; i < 8; i++, j--) {
            reversed[i] = bytes[j];
        }
        // 使用BigInteger的构造函数，避免负数
        return new BigInteger(1, reversed).longValue();
    }
    public static long bytesToUInt32LE(byte[] bytes) {
        if (bytes == null || bytes.length < 4) {
            throw new IllegalArgumentException("The byte array must not be null and must be at least 4 bytes long.");
        }
        ByteBuffer buffer = ByteBuffer.allocate(4);
        buffer.put(bytes, 0, 4);
        buffer.flip();
        return buffer.getInt() & 0xFFFFFFFFL; // 转换为无符号整数
    }
    public static int bytesToUInt16LE(byte[] bytes) {
        if(bytes == null || bytes.length < 2) {
            throw new IllegalArgumentException("The byte array must not be null and must contain at least 2 bytes.");
        }

        ByteBuffer buffer = ByteBuffer.allocate(2);
        buffer.put(bytes[0]);
        buffer.put(bytes[1]);
        buffer.flip();

        return buffer.getShort() & 0xFFFF;
    }
    public static int bytesToUInt8LE(byte b) {
        return b & 0xFF; // 将byte转换为int，并确保为正数
    }

    //byteToBytes
    public static byte[] uint8ToBytes(byte uint8) {
        return new byte[]{ uint8 };
    }
    //shortToBytes
    public static byte[] uint16ToBytes(short number) {
        byte[] bytes = new byte[2];
        bytes[0] = (byte) (number >> 8); // 高位字节
        bytes[1] = (byte) (number & 0xFF); // 低位字节
        return bytes;
    }
    //intToBytes
    public static byte[] uint32ToBytes(int x) {
        byte[] bytes = new byte[4];
        ByteBuffer.wrap(bytes).putInt(x);
        return bytes;
    }
    //longToBytes
    public static byte[] uint64ToBytes(long value) {
        ByteBuffer buffer = ByteBuffer.allocate(8);
        buffer.putLong(value);
        return buffer.array();
    }
    public static byte[] uint64ToBytesLE(long value) {
        ByteBuffer buffer = ByteBuffer.allocate(8);
        buffer.putLong(value);
        buffer.order(ByteOrder.LITTLE_ENDIAN);
        return buffer.array();
    }
    public static byte[] getRandomUint32(){
        Random random = new Random();
        int random6DigitInt = 10000000 + random.nextInt(90000000);
        return uint32ToBytes(random6DigitInt);
    }
    public static String printByte(byte[] bytes){
        String result = "" ;
        for (byte b : bytes) {
            result +=b+", ";
        }
        result = result.substring(0,result.lastIndexOf(","));
        result = "{"+result+"}";
        return result;
    }

    public static byte[] toLittleEndian(String ip) throws Exception {
        InetAddress address = InetAddress.getByName(ip);
        byte[] bytes = address.getAddress();

        // 如果JVM是小端的，则不需要做任何操作
        if (!ByteOrder.nativeOrder().equals(ByteOrder.LITTLE_ENDIAN)) {
            // 反转字节数组以得到小端表示
            for (int i = 0; i < bytes.length / 2; i++) {
                byte temp = bytes[i];
                bytes[i] = bytes[bytes.length - 1 - i];
                bytes[bytes.length - 1 - i] = temp;
            }
        }

        return bytes;
    }

    /**
     * 传入Data类型日期，返回字符串类型时间（ISO8601标准时间）
     * @param date
     * @return
     */
    public static String getISO8601Timestamp(Date date){
        TimeZone tz = TimeZone.getTimeZone("UTC");
        DateFormat df = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSS'Z'");
        df.setTimeZone(tz);
        String nowAsISO = df.format(date);
        String fill = "";
        for (int i = 0;i < 32 - nowAsISO.length();i++){
            fill += 0;
        }
        return nowAsISO += fill;
    }

    public static byte[] convertStringToByteArray(String str, int targetLength) {
        byte[] bytes = str.getBytes();
        // 如果字节数组长度已经是16，则直接返回
        if (bytes.length == targetLength) {
            return bytes;
        }
        // 创建一个新的字节数组，长度为16
        byte[] result = new byte[targetLength];
        // 将原始字节数组复制到新数组中
        System.arraycopy(bytes, 0, result, 0, Math.min(bytes.length, targetLength));
        // 如果原始字节数组长度小于16，则填充剩余部分
        if (bytes.length < targetLength) {
            // 这里使用0填充剩余的字节，也可以根据需要填充其他字节值
            Arrays.fill(result, bytes.length, targetLength, (byte) 0);
        }
        return result;
    }

    public static String byteToString(byte[] bytes)throws Exception{
        String str = new String(bytes,"UTF-8");
        return str;
    }

    public static String addSpaces(String original, int interval) {
        if (original == null || original.isEmpty() || interval <= 0) {
            return original;
        }

        StringBuilder modified = new StringBuilder();
        for (int i = 0; i < original.length(); i++) {
            modified.append(original.charAt(i));
            if (i % interval == interval - 1) {
                modified.append(" ");
            }
        }
        return modified.toString();
    }
    public static String addCodePoint(String str){
        StringBuilder sb = new StringBuilder();
        sb.append(str);
        sb.appendCodePoint(0);
        return sb.toString();
    }

    public static byte[] stringToByte(String str,int length){
        byte[] bytes = str.getBytes(StandardCharsets.UTF_8);
        if (bytes.length < length) {
            byte[] paddedBytes = new byte[length];
            System.arraycopy(bytes, 0, paddedBytes, 0, bytes.length);
            bytes = paddedBytes;
        }
        return bytes;
    }

    public static byte[] stringToByte2(String str,int length){
        byte[] bytes = str.getBytes(StandardCharsets.UTF_8);
        if (bytes.length < length) {
            byte[] fixedBytes = new byte[length];
            System.arraycopy(bytes, 0, fixedBytes, length - bytes.length, bytes.length);
            bytes = fixedBytes;
        }
        return bytes;
    }

    public static void main(String[] args) {

        String a = getISO8601Timestamp(new Date());
        System.out.println(a);

        // 示例：使用byte数组表示Uint16, Uint32, Uint64
        byte[] uint8Bytes = {10};
        byte uint8 = toUint8(uint8Bytes);
        byte[] uint16Bytes = {0, 10}; // 0x00, 0x0A
        short uint16 = toUint16(uint16Bytes);

        byte[] uint32Bytes = {0, 0, 0, 10}; // 0x00, 0x00, 0x00, 0x0A
        int uint32 = toUint32(uint32Bytes);

        byte[] uint64Bytes = {0, 0, 0, 0, 0, 0, 0, 10}; // 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0A
        long uint64 = toUint64(uint64Bytes);

        short ms = 10;
        byte[] byteBytes = uint8ToBytes(Byte.valueOf("10"));
        byte[] shortBytes = uint16ToBytes(ms);
        byte[] intBytes = uint32ToBytes(10);
        byte[] longBytes = uint64ToBytes(10);

        System.out.println("uint8: "+uint8+" ,uint16: " + uint16+" ,uint32: "+uint32+" ,uint64: "+uint64);
        System.out.println("byteBytes: " + printByte(byteBytes)+" ,shortBytes: " + printByte(shortBytes)+" ,intBytes: "+printByte(intBytes)+" ,longBytes: "+printByte(longBytes));

        String splUuid = "291beb29404e4a299266ce765c107d18";
        byte[] bytes = new byte[16];
        System.arraycopy(splUuid.getBytes(), 0, bytes, 0, 16);
        System.out.println(printByte(bytes));
    }
}
//        UInt64    无符号 64 位整数    0 到 18446744073709551615   8 字节   long
//        UIn32     无符号 32 位整数    0 到 4294967295             4 字节   int
//        UInt16    无符号 16 位整数    0 到 65535                  2 字节   short
//        UInt8     无符号8位整数       0 到 255                    1 字节   byte