package com.miaozhen.device.support.util;

import org.apache.http.HttpResponse;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.ResponseHandler;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.config.Registry;
import org.apache.http.config.RegistryBuilder;
import org.apache.http.conn.socket.ConnectionSocketFactory;
import org.apache.http.conn.socket.PlainConnectionSocketFactory;
import org.apache.http.conn.ssl.NoopHostnameVerifier;
import org.apache.http.conn.ssl.SSLConnectionSocketFactory;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.impl.conn.PoolingHttpClientConnectionManager;
import org.apache.http.ssl.SSLContexts;
import org.apache.http.ssl.TrustStrategy;
import org.apache.http.util.EntityUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.net.ssl.SSLContext;
import java.io.IOException;
import java.security.KeyManagementException;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;

/**
 * @Description :
 * Created by liyongbo on 2024-12-05 17:03.
 */
public class SSLHttpClientUtil {

    private static final Logger logger = LoggerFactory.getLogger(SSLHttpClientUtil.class);

    public static PoolingHttpClientConnectionManager connectionManager = null;//这是一个静态的连接池管理器，用于管理HTTP连接的池
    private static RequestConfig requestConfig;//用于设置HTTP请求的参数，比如超时时间等
    public static CloseableHttpClient client;//这是实际用于发送HTTP请求的客户端实例。

    //单例模式创建
    private static void init(){
        synchronized (SSLHttpClientUtil.class) {
            if (client == null) {
                SSLContext sslContext = null;
                try {
                    sslContext = SSLContexts.custom()
                            .loadTrustMaterial(new TrustStrategy() {
                                @Override
                                public boolean isTrusted(X509Certificate[] chain, String authType) throws CertificateException {
                                    return true;
                                }
                            })
                            .build();
                } catch (NoSuchAlgorithmException e) {
                    e.printStackTrace();
                } catch (KeyManagementException e) {
                    e.printStackTrace();
                } catch (KeyStoreException e) {
                    e.printStackTrace();
                }
                SSLConnectionSocketFactory sslsf = new SSLConnectionSocketFactory(
                        sslContext,
                        NoopHostnameVerifier.INSTANCE);
                PlainConnectionSocketFactory plainConnectionSocketFactory = new PlainConnectionSocketFactory();
                Registry<ConnectionSocketFactory> registry = RegistryBuilder.<ConnectionSocketFactory>create()
                        .register("https", sslsf)
                        .register("http", plainConnectionSocketFactory)
                        .build();
                connectionManager = new PoolingHttpClientConnectionManager(registry);

                // 创建请求配置
                requestConfig = RequestConfig.custom()
                        .setConnectTimeout(5000) // 连接超时时间5秒e
                        .setSocketTimeout(10000)  // 数据读取超时时间10秒r
                        .setConnectionRequestTimeout(5000) // 连接不受限的超时时间5秒
                        .build();

                ResponseHandler<String> handler = new ResponseHandler<String>() {
                    @Override
                    public String handleResponse(HttpResponse response) throws ClientProtocolException, IOException {
                        return EntityUtils.toString(response.getEntity(), "UTF-8");
                    }
                };

                client = HttpClients.custom()
                        .setConnectionManager(connectionManager)
                        .setDefaultRequestConfig(requestConfig)
                        .build();

            }
        }
    }


    /**
     * 从池子中获取连接
     * @return CloseableHttpClient
     */
    public static CloseableHttpClient getClientFromHttpPool() {
        if(client == null) {
            init();
        }
        return client;
    }

    public static void shutdown(){
        if(client !=null){
            try {
                client.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        if(connectionManager != null){
            connectionManager.shutdown();
        }
    }

    public static void restart(){
        logger.info("LedServer sslHttpClient restart");
        shutdown();
        init();
        logger.info("LedServer sslHttpClient restart over");
    }

}
