package com.miaozhen.util;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

public class ExpiringCacheUtil<K, V> {
    private final Map<K, V> cache = new ConcurrentHashMap<>();
    private final ScheduledExecutorService scheduler = Executors.newScheduledThreadPool(1);
    private final int expirationSeconds;

    public ExpiringCacheUtil(int expirationSeconds) {
        this.expirationSeconds = expirationSeconds;
    }

    public void put(K key, V value) {
        cache.put(key, value);
        scheduleExpiration(key, expirationSeconds);
    }

    public V get(K key) {
        return cache.get(key);
    }

    private void scheduleExpiration(K key, int seconds) {
        scheduler.schedule(() -> cache.remove(key), seconds, TimeUnit.SECONDS);
    }

    public void shutdown() {
        scheduler.shutdown();
    }

    public static void main(String[] args) {
        ExpiringCacheUtil<String,String> cacheUtil = new ExpiringCacheUtil<>(10);
        cacheUtil.put("key1", "value1");
        String value = cacheUtil.get("key1");
    }
}
