package com.miaozhen.util;

import com.miaozhen.Application;
import com.miaozhen.tmssys.dto.CinemaInfo;
import com.miaozhen.tmssys.dto.PlaybackStatus;
import com.miaozhen.tmssys.dto.ReportLog;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.system.ApplicationHome;

import java.io.*;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.List;
import java.util.concurrent.ConcurrentHashMap;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;

public class FileUtil {

    private static Logger logger = LoggerFactory.getLogger(FileUtil.class);

    /**
     * 解压缩ZIP文件
     * @param zipFile ZIP文件
     * @param destDir 目标路径
     */
    public static void zipDecompress(File zipFile, File destDir) {
        byte[] buffer = new byte[1024];
        try{
            ZipInputStream zis = new ZipInputStream(new FileInputStream(zipFile));
            ZipEntry entry = zis.getNextEntry();
            while (entry != null) {
                File file = new File(destDir, entry.getName());
                if (entry.isDirectory()) {
                    file.mkdirs();
                } else {
                    File parent = file.getParentFile();
                    if (!parent.exists()) {
                        parent.mkdirs();
                    }
                    try (FileOutputStream fos = new FileOutputStream(file)) {
                        int len;
                        while ((len = zis.read(buffer)) > 0) {
                            fos.write(buffer, 0, len);
                        }
                    }
                }
                entry = zis.getNextEntry();
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public static String readFileToString(File file) throws IOException {

        BufferedReader reader = new BufferedReader(new FileReader(file));
        StringBuilder stringBuilder = new StringBuilder();

        String line;
        while ((line = reader.readLine()) != null) {
            stringBuilder.append(line).append("\n");
        }
        reader.close();
        return stringBuilder.toString();
    }

    public static void checkFolder(String sPath) {
        // 指定目录路径
        Path path = Paths.get(sPath);

        // 判断目录是否存在，不存在则创建
        if (!Files.exists(path)) {
            try {
                Files.createDirectories(path); // 创建目录（包括父目录）
                System.out.println("目录创建成功: " + path);
            } catch (IOException e) {
                System.err.println("目录创建失败: " + e.getMessage());
            }
        } else {
            System.out.println("目录已存在: " + path);
        }
    }

    public static String getFTPPath(){
        String path = "";
        if(OSinfoUtil.isLinux()){
            path = "/tmp/ftp";
        }else{
            path = "D:\\tmp\\ftp";
        }
        return path;
    }

    public static String getAudtiInfosPath(){
        String path = "";
        if(OSinfoUtil.isLinux()){
            path = "/home/smart/.tms3/";
        }else{
            path = "D:\\work\\bbt\\";
        }
        return path + "AuditInfos.txt";
    }


   public static boolean deleteFolder(String sPath) {
        boolean flag = false;
        File file = new File(sPath);
        // 判断目录或文件是否存在
        if (!file.exists()) {  // 不存在返回 false
            return flag;
        } else {
            // 判断是否为文件
            if (file.isFile()) {  // 为文件时调用删除文件方法
                return deleteFile(sPath);
            } else {  // 为目录时调用删除目录方法
                return deleteDirectory(sPath);
            }
        }
    }

    /**
     * 删除单个文件
     *
     * @param sPath 被删除文件的文件名
     * @return 单个文件删除成功返回true，否则返回false
     */
    public static boolean deleteFile(String sPath) {
        boolean flag = false;
        File file = new File(sPath);
        // 路径为文件且不为空则进行删除
        if (file.isFile() && file.exists()) {
            file.delete();
            flag = true;
        }
        return flag;
    }


    /**
     * 删除目录（文件夹）以及目录下的文件
     *
     * @param sPath 被删除目录的文件路径
     * @return 目录删除成功返回true，否则返回false
     */
    public static boolean deleteDirectory(String sPath) {
        //如果sPath不以文件分隔符结尾，自动添加文件分隔符
        if (!sPath.endsWith(File.separator)) {
            sPath = sPath + File.separator;
        }
        File dirFile = new File(sPath);
        //如果dir对应的文件不存在，或者不是一个目录，则退出
        if (!dirFile.exists() || !dirFile.isDirectory()) {
            return false;
        }
        boolean flag = true;
        //删除文件夹下的所有文件(包括子目录)
        File[] files = dirFile.listFiles();
        for (int i = 0; i < files.length; i++) {
            //删除子文件
            if (files[i].isFile()) {
                flag = deleteFile(files[i].getAbsolutePath());
                if (!flag) break;
            } //删除子目录
            else {
                flag = deleteDirectory(files[i].getAbsolutePath());
                if (!flag) break;
            }
        }
        if (!flag) return false;
        //删除当前目录
        if (dirFile.delete()) {
            return true;
        } else {
            return false;
        }
    }

    public static boolean isHave(String sPath) {
        File file = new File(sPath.substring(0, sPath.lastIndexOf(".")));
        // 判断目录或文件是否存在
        if (file.isDirectory() && file.exists()) {  // 不存在返回 false
            return true;
        }
        return false;
    }

    public static void setDirPermissions(String strPath) {
        try {
            Runtime.getRuntime().exec("chmod -R 777  " + strPath);
        } catch (IOException e) {
        }
    }

    /**
     * 获取项目产生日志的目录
     * @return
     */
    public static String getLogsDir() {
        ApplicationHome home = new ApplicationHome(Application.class);
        File jarFile = home.getSource();
        String path = jarFile.getParentFile().getPath();
        String tempFilePath = path + File.separator + "logs";
        return tempFilePath;
    }
    /**
     * 获取创建的文件夹serverlogs的临时目录
     * @return
     */
    public static String getServerlogsDir() {
        ApplicationHome home = new ApplicationHome(Application.class);
        File jarFile = home.getSource();
        String path = jarFile.getParentFile().getPath();
        String tempFilePath = path + File.separator + "serverlogs";
        File file = new File(tempFilePath);
        if (!file.exists()) {
            file.mkdir();
        }
        return tempFilePath;
    }
    /**
     * 获取创建的文件夹的临时目录
     * @return
     */
    public static String getTempLogDir() {
        ApplicationHome home = new ApplicationHome(Application.class);
        File jarFile = home.getSource();
        String path = jarFile.getParentFile().getPath();
        String tempFilePath = path + File.separator + "tempLog";
        File file = new File(tempFilePath);
        if (!file.exists()) {
            file.mkdir();
        }
//        setDirPermissions(tempFilePath);
        return tempFilePath;
    }

    /**
     * 将每天日志上报记录写到本地缓存文件
     *
     * @param map
     */
    public synchronized static void writeReportLogToTemp(ConcurrentHashMap<String, ReportLog> map) {
        String path = getServerlogsDir() + File.separator + "ReportLog";
        FileOutputStream fos = null;
        ObjectOutputStream os = null;
        try {
            File f = new File(path);
            if (!f.exists()) {
                f.createNewFile();
            }
            fos = new FileOutputStream(path);
            os = new ObjectOutputStream(fos);
            os.writeObject(map);
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                if (fos != null) {
                    fos.close();
                }
                if (os != null) {
                    os.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * 将影院相关信息CinemaInfo写到本地缓存文件
     *
     * @param cinemaInfo
     */
    public synchronized static void writeCinemaInfoToTemp(CinemaInfo cinemaInfo) {
        String path = getServerlogsDir() + File.separator + "CinemaInfo";
        FileOutputStream fos = null;
        ObjectOutputStream os = null;
        try {
            File f = new File(path);
            if (!f.exists()) {
                f.createNewFile();
            }
            fos = new FileOutputStream(path);
            os = new ObjectOutputStream(fos);
            os.writeObject(cinemaInfo);
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                if (fos != null) {
                    fos.close();
                }
                if (os != null) {
                    os.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * 从临时文件获取每天日志上报记录
     *
     * @return
     */
    public static ConcurrentHashMap<String, ReportLog> getReportLogFromTemp() {
        ConcurrentHashMap<String, ReportLog> result = new ConcurrentHashMap();
        String path = getServerlogsDir() + File.separator + "ReportLog";
        if (!new File(path).exists()) {
            return result;
        }
        ObjectInputStream is = null;
        FileInputStream fis = null;
        try {
            fis = new FileInputStream(path);
            is = new ObjectInputStream(fis);
            result = (ConcurrentHashMap<String, ReportLog>) is.readObject();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (is != null) {
                try {
                    is.close();
                } catch (IOException e1) {
                }
            }
            if (fis != null) {
                try {
                    fis.close();
                } catch (IOException e1) {
                }
            }
        }
        return result;
    }

    /**
     * 从临时文件获取影院相关信息CinemaInfo
     *
     * @return
     */
    public static CinemaInfo getCinemaInfoFromTemp() {
        CinemaInfo result = null;
        String path = getServerlogsDir() + File.separator + "CinemaInfo";
        if (!new File(path).exists()) {
            return result;
        }
        ObjectInputStream is = null;
        FileInputStream fis = null;
        try {
            fis = new FileInputStream(path);
            is = new ObjectInputStream(fis);
            result = (CinemaInfo) is.readObject();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (is != null) {
                try {
                    is.close();
                } catch (IOException e1) {
                }
            }
            if (fis != null) {
                try {
                    fis.close();
                } catch (IOException e1) {
                }
            }
        }
        return result;
    }
    /**
     * 从临时文件获取影院相关信息CinemaInfo  【测试使用】
     *
     * @return
     */
    public static CinemaInfo getCinemaInfoTestFromTemp() {
        CinemaInfo result = null;
        String path = "D:\\serverlogs\\CinemaInfo";
        if (!new File(path).exists()) {
            return result;
        }
        ObjectInputStream is = null;
        FileInputStream fis = null;
        try {
            fis = new FileInputStream(path);
            is = new ObjectInputStream(fis);
            result = (CinemaInfo) is.readObject();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (is != null) {
                try {
                    is.close();
                } catch (IOException e1) {
                }
            }
            if (fis != null) {
                try {
                    fis.close();
                } catch (IOException e1) {
                }
            }
        }
        return result;
    }
    /**
     * 将影厅服务器状态PlaybackStatus写到本地缓存文件  【测试使用】
     *
     * @param list
     */
    public synchronized static void writePlaybackStatusToTemp(List<PlaybackStatus> list) {
        String path = getServerlogsDir() + File.separator + "PlaybackStatus";
        FileOutputStream fos = null;
        ObjectOutputStream os = null;
        try {
            File f = new File(path);
            if (!f.exists()) {
                f.createNewFile();
            }
            fos = new FileOutputStream(path);
            os = new ObjectOutputStream(fos);
            os.writeObject(list);
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                if (fos != null) {
                    fos.close();
                }
                if (os != null) {
                    os.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }
    /**
     * 从临时文件获取PlaybackStatus  【测试使用】
     *
     * @return
     */
    public static List<PlaybackStatus> getPlaybackStatusFromTemp() {
        List<PlaybackStatus> result = null;
//        String path ="D:\\serverlogs\\PlaybackStatus";
        String path = getServerlogsDir() + File.separator + "PlaybackStatus";
        if (!new File(path).exists()) {
            return result;
        }
        ObjectInputStream is = null;
        FileInputStream fis = null;
        try {
            fis = new FileInputStream(path);
            is = new ObjectInputStream(fis);
            result = (List<PlaybackStatus>) is.readObject();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (is != null) {
                try {
                    is.close();
                } catch (IOException e1) {
                }
            }
            if (fis != null) {
                try {
                    fis.close();
                } catch (IOException e1) {
                }
            }
        }
        return result;
    }

    /**
     * 从缓存xml中读取信息并转换成对象
     * @return
     * @throws Exception
     */
    public static CinemaInfo getCinemaInfoFromXml() {
        try {
            String path = getServerlogsDir() + File.separator + "CinemaInfo.xml";
            File file = new File(path);
            String xml = "";
            if(file.exists()){
                ByteArrayOutputStream bos=new ByteArrayOutputStream();
                FileInputStream fis=new FileInputStream(file);
                int a=0;
                while((a=fis.read())>0){
                    bos.write(a);
                }
                xml=new String(bos.toByteArray());
            }
            if(StringUtils.isNotBlank(xml)){
                return JaxbUtil.converyToJavaBean(xml,CinemaInfo.class);
            }
        }catch (Exception e){
            e.printStackTrace();
        }
        return null;
    }

    public static String getCinemaInfoMd5() {
        String md5 = "";
        try {
            String path = getServerlogsDir() + File.separator + "CinemaInfo.xml";
            File file = new File(path);
            if (file.exists()) {
                md5 = DigestUtils.md5Hex(new FileInputStream(file));
                System.out.println("config md5 = " + md5);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return md5;
    }

    /**
     * 把对象信息转换xml并写入缓存文件中
     * @param cinemaInfo
     * @throws Exception
     */
    public static void setCinemaInfoToXml(CinemaInfo cinemaInfo){
        try {
            String path = getServerlogsDir() + File.separator + "CinemaInfo.xml";
            String xml = JaxbUtil.convertToXml(cinemaInfo,"UTF-8");//转换xml
            FileOutputStream fos = new FileOutputStream(path);
            fos.write(xml.getBytes());
        }catch (Exception e){
            e.printStackTrace();
        }

    }

    /**
     * 是否上报了2019-12-01之后的所有日志记录
     * @param map
     */
    public synchronized static void writeIsReportHistoryLogToTemp(ConcurrentHashMap<String, Boolean> map) {
        String path = getServerlogsDir() + File.separator + "IsReportHistoryLog";
        FileOutputStream fos = null;
        ObjectOutputStream os = null;
        try {
            File f = new File(path);
            if (!f.exists()) {
                f.createNewFile();
            }
            fos = new FileOutputStream(path);
            os = new ObjectOutputStream(fos);
            os.writeObject(map);
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                if (fos != null) {
                    fos.close();
                }
                if (os != null) {
                    os.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * 获取是否上报完毕2019-12-01之后的所有日志记录标识
     * @return
     */
    public static ConcurrentHashMap<String, Boolean> getIsReportHistoryLogFromTemp() {
        ConcurrentHashMap<String, Boolean> result = new ConcurrentHashMap();
        String path = getServerlogsDir() + File.separator + "IsReportHistoryLog";
        if (!new File(path).exists()) {
            return result;
        }
        ObjectInputStream is = null;
        FileInputStream fis = null;
        try {
            fis = new FileInputStream(path);
            is = new ObjectInputStream(fis);
            result = (ConcurrentHashMap<String, Boolean>) is.readObject();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (is != null) {
                try {
                    is.close();
                } catch (IOException e1) {
                }
            }
            if (fis != null) {
                try {
                    fis.close();
                } catch (IOException e1) {
                }
            }
        }
        return result;
    }
    public static void main(String[] args) {
    }

    public static void createPathIfNotExists(String path) {
        Path filePath = Paths.get(path);

        // 检查路径是否存在
        if (!Files.exists(filePath)) {
            try {
                // 创建父目录（如果不存在）
                Path parentDir = filePath.getParent();
                if (parentDir != null && !Files.exists(parentDir)) {
                    Files.createDirectories(parentDir);
                    System.out.println("父目录创建成功: " + parentDir.toAbsolutePath());
                }

                // 创建文件或目录
                if (path.endsWith("/") || path.endsWith("\\")) {
                    // 如果路径以斜杠结尾，创建目录
                    Files.createDirectory(filePath);
                    System.out.println("目录创建成功: " + filePath.toAbsolutePath());
                } else {
                    // 创建文件
                    Files.createFile(filePath);
                    System.out.println("文件创建成功: " + filePath.toAbsolutePath());
                }
            } catch (IOException e) {
                System.out.println("路径创建异常: " + e.getMessage());
            }
        } else {
            FileUtil.deleteFile(path);
            createPathIfNotExists(path);
        }
    }

    public static String read(String path) {
        FileInputStream is = null;
        ObjectInputStream ois = null;
        String result = "";
        try {
            File f = new File(path);
            if (!f.exists()) {
                return result;
            }
            is = new FileInputStream(path);
            ois = new ObjectInputStream(is);
            result = ois.readObject().toString();
        } catch (IOException e) {
            e.printStackTrace();
        } catch (ClassNotFoundException e) {
            e.printStackTrace();
        } finally {
            try {
                if (is != null) {
                    is.close();
                }
                if (ois != null) {
                    ois.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return result;
    }
}

